/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: LogAsyncMessage.java,v 1.11 2005/02/16 22:21:33 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.sd.logc.internal.uml2sd;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.hyades.loaders.cbe.CBEUtils;
import org.eclipse.hyades.log.ui.internal.LogUIPlugin;
import org.eclipse.hyades.log.ui.internal.util.LogUIConstants;
import org.eclipse.hyades.models.cbe.CBECommonBaseEvent;
import org.eclipse.hyades.models.hierarchy.extensions.QueryResult;
import org.eclipse.hyades.models.hierarchy.extensions.ResultEntry;
import org.eclipse.hyades.models.hierarchy.impl.CorrelationEntryImpl;
import org.eclipse.hyades.sd.logc.SDLogcConstants;
import org.eclipse.hyades.sd.logc.SDLogcPlugin;
import org.eclipse.hyades.sd.logc.internal.loader.LogInteractions;
import org.eclipse.hyades.trace.ui.internal.util.TString;
import org.eclipse.hyades.uml2sd.ui.core.AsyncMessage;
import org.eclipse.hyades.uml2sd.ui.load.LoadersManager;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;


public class LogAsyncMessage extends AsyncMessage implements IPropertySource {
	
	public class SourceTargetPair {
		private CBECommonBaseEvent sourceEvent;
		private CBECommonBaseEvent targetEvent;
		
		public SourceTargetPair(CBECommonBaseEvent source, CBECommonBaseEvent target) {
			setSourceEvent(source);
			setTargetEvent(target);
		}
		
		public CBECommonBaseEvent getSourceEvent() {
			return sourceEvent;
		}

		public void setSourceEvent(CBECommonBaseEvent sourceEvent) {
			this.sourceEvent = sourceEvent;
		}

		public CBECommonBaseEvent getTargetEvent() {
			return targetEvent;
		}

		public void setTargetEvent(CBECommonBaseEvent targetEvent) {
			this.targetEvent = targetEvent;
		}
	}
	
	private List correlationList = null;
	private LogGraphNode sourceNode = null;
	private LogGraphNode targetNode = null;
	private int type;
	
	public LogAsyncMessage(int type){
		this.type = type;
	}
	
	public void setSourceNode(LogGraphNode sourceNode){
		this.sourceNode=sourceNode;
	}
	
	public void setTargetNode(LogGraphNode targetNode) {
		this.targetNode = targetNode;
	}
	
	public void addPair(CBECommonBaseEvent source, CBECommonBaseEvent target) {
		SourceTargetPair pair = new SourceTargetPair(source,target);
		correlationList.add(pair);	
	}
	
	public List getPairs(){
		return correlationList;
	}

	public void generatePairs() {
		
		//no need to generate again if the list is already generated
		if (correlationList!=null) return;

		correlationList = new ArrayList();
		
		List sourceElements = null; 
		if(sourceNode.hasElements() && !sourceNode.isExpanded()){
			sourceElements = sourceNode.getElements();
		}else{
			sourceElements = new ArrayList(1);
			sourceElements.add(sourceNode.getElement());
		}
		
		List targetElements = null;
		if(targetNode.hasElements() && !targetNode.isExpanded()){
			targetElements = targetNode.getElements();
		}else{
			targetElements = new ArrayList(1);
			targetElements.add(targetNode.getElement());			
		}
		
		if(type==LogUIConstants.XMI){	
			generateCorrelationListXMI(sourceElements, targetElements);
		}else if(type==LogUIConstants.SQL){
			generateCorrelationListSQL(sourceElements, targetElements);
		}
			

	}
	
	/**
	 * @param sourceElements
	 * @param targetElements
	 */
	private void generateCorrelationListSQL(List sourceElements, List targetElements) {
		
		LogInteractions loader = (LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID);
		QueryResult correlations = loader.getCorrelations();
		ResultEntry resultEntry =(ResultEntry)correlations.getResultEntries().get(0);
		List keysList = (List)resultEntry.getValue();
		resultEntry =(ResultEntry)correlations.getResultEntries().get(1);
		List valuesList = (List)resultEntry.getValue();
		
		if(keysList.size()>0){
			CBECommonBaseEvent source = (CBECommonBaseEvent)sourceElements.get(0);
			CBECommonBaseEvent target = null;
			int startIndex = keysList.indexOf(source);
			source = (CBECommonBaseEvent)keysList.get(sourceElements.size()-1);
			int s = keysList.size();			
			for(int i=startIndex;i<s;i++){
				source = (CBECommonBaseEvent)keysList.get(i);
				target = (CBECommonBaseEvent)valuesList.get(i);
				if(!sourceElements.contains(source))
					break;
				if(!targetElements.contains(target)){
					continue;
				}
				addPair(source,target);
			}
		}		
	}

	private void generateCorrelationListXMI(List sourceElements, List targetElements) {
		CBECommonBaseEvent source = null;
		List targetList = null;

		CorrelationEntryImpl entry = null;
		Object key = null;
 
		LogInteractions loader = (LogInteractions)LoadersManager.getLoadersManager().getCurrentLoader(SDLogcConstants.LogInteractionsViewID);
		EMap correlations  = loader.getCorrelationContainerProxy().getCorrelationContainer().getCorrelations();
		
		CBECommonBaseEvent target;
		for (Iterator iter = correlations.iterator(); iter.hasNext();) {
			entry = (CorrelationEntryImpl) iter.next();
			key = entry.getKey();
			
			if(!(key instanceof CBECommonBaseEvent)){
				continue;
			}
			
			source = (CBECommonBaseEvent)key;
			if (!sourceElements.contains(source)) continue;
			
			targetList = (List)entry.getValue();
			if(targetList==null)
				continue;
			int s = targetList.size();
			for(int i = 0;i < s;i++){
				target = (CBECommonBaseEvent)targetList.get(i);
				if (!targetElements.contains(target)) continue;
				addPair(source,target);
			}
		}
	}

	private IPropertyDescriptor[] propertyDescriptors;
	private static final String PROPERTY_LIST = "correlation.list."; 	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		generatePairs();
		
		if (correlationList.size()==1) {
			if (propertyDescriptors == null) {
				propertyDescriptors = new IPropertyDescriptor[1];
				PropertyDescriptor cbeListsDescriptor = 
					new PropertyDescriptor(PROPERTY_LIST, "["+CBEUtils.formatCreationTime(((SourceTargetPair)correlationList.get(0)).getSourceEvent(),LogUIPlugin.getDateTimeFormat())+"] "+((SourceTargetPair)correlationList.get(0)).getSourceEvent().getMsg());
					cbeListsDescriptor.setLabelProvider(new LabelProvider() {
						   public String getText(Object element) {
							return  "--> ["+CBEUtils.formatCreationTime(((SourceTargetPair)correlationList.get(0)).getTargetEvent(),LogUIPlugin.getDateTimeFormat())+"] "+((SourceTargetPair)correlationList.get(0)).getTargetEvent().getMsg();
						   }
					});
				cbeListsDescriptor.setCategory(TString.change(SDLogcPlugin.getString("STR_LOG_CORR"),"%1",""+correlationList.size()));
					propertyDescriptors[0]=cbeListsDescriptor;
			}
		}
		else {
			if (propertyDescriptors == null) {
				propertyDescriptors = new IPropertyDescriptor[correlationList.size()];
				for (int i=0;i<correlationList.size();i++) {
					PropertyDescriptor cbeListsDescriptor = 
						new PropertyDescriptor(PROPERTY_LIST+i, "["+CBEUtils.formatCreationTime(((SourceTargetPair)correlationList.get(i)).getSourceEvent(),LogUIPlugin.getDateTimeFormat())+"] "+((SourceTargetPair)correlationList.get(i)).getSourceEvent().getMsg());
					final int idx = i;
					cbeListsDescriptor.setLabelProvider(new LabelProvider() {
						   public String getText(Object element) {
							return  "--> ["+CBEUtils.formatCreationTime(((SourceTargetPair)correlationList.get(idx)).getTargetEvent(),LogUIPlugin.getDateTimeFormat())+"] "+((SourceTargetPair)correlationList.get(idx)).getTargetEvent().getMsg();
						   }
						});

					cbeListsDescriptor.setCategory(TString.change(SDLogcPlugin.getString("STR_LOG_CORRS"),"%1",""+correlationList.size()));
					propertyDescriptors[i]=cbeListsDescriptor;
				}
			}
		}
        return propertyDescriptors;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		   if (((String)id).indexOf(PROPERTY_LIST)==0) {
		   	return correlationList;
		   }
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {	}
	/**
	 * @return
	 */
	public LogGraphNode getSourceNode() {
		return sourceNode;
	}

	/**
	 * @return
	 */
	public LogGraphNode getTargetNode() {
		return targetNode;
	}

}
