/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.statistical.ui;

import java.lang.reflect.Method;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Platform;
import org.eclipse.hyades.model.statistical.impl.StatisticalPackageImpl;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The main plugin class to be used in the desktop.
 */
public class EditorPlugin extends AbstractUIPlugin {

	public static final String PACKAGE_DEBUGGER = "org.eclipse.hyades.statistical.ui";
	public static final String PACKAGE_STATCON_MODULE = "org.eclipse.hyades.statistical.ui.StatConModule";
	public static final String PACKAGE_STATCON_FILE = "org.eclipse.hyades.statistical.ui.StatConFile";
	public static final String PACKAGE_STATCON_STAT_IMG_PROVIDER = "org.eclipse.hyades.statistical.ui.StatConStatisticalImageProvider";

	//The shared instance.
	private static EditorPlugin plugin;

	public static Debugger DBG = new Debugger(PACKAGE_DEBUGGER);

	//Resource bundle.
	private ResourceBundle resourceBundle;

	public static ImageManager img = new ImageManager();
	
	public static void disposeObject(Object o) throws Throwable {
		Class c = o.getClass();

		Throwable error = null;
							
		while (c != null) {				
			try {
				Method o_dispose = c.getDeclaredMethod("dispose",new Class[]{});
				o_dispose.setAccessible(true);
				o_dispose.invoke(o,new Object[]{});
				error = null;
				break;
			} catch (Throwable e) { 
				error = e;
			}
			c = c.getSuperclass();
		}
		
		if (error != null) throw error;
	}

	/**
	 * The constructor.
	 */
	public EditorPlugin() {
//		super(descriptor);
		plugin = this;
		DBG.setPlugin(this);
		
		EditorPlugin.DBG.info("StatCon Plugin Constructor");

		EditorPlugin.DBG.info("calling StatisticalPackageImpl.init() to fix bugs");
		StatisticalPackageImpl.init();

		img.initialize("icons/full/",this,"org.eclipse.hyades.statistical.ui");
		
/*
		try {
			img.initialize(new URL(getDescriptor().getInstallURL(),"icons/full/"), ImageRegistryProvider.createImageRegistry());
			img.addImages();
		} catch (Exception e) {
			e.printStackTrace();
		}
*/		
/*
		try {
			EditorPlugin.DBG.info("initializing images");
			img.initialize(new URL(getDescriptor().getInstallURL(),"icons/full/"), getImageRegistry());

			img.addImages();
		} catch (Exception e) {
			e.printStackTrace();
		}
*/		
//		resourceBundle = this.getDescriptor().getResourceBundle();

//		plugin = this;

//		resourceBundle = getResourceBundle();
//		resourceBundle = Platform.getResourceBundle(getBundle());

	}

	protected void initializeImageRegistry(ImageRegistry reg)
	{
//		img.initialize(this,"org.eclipse.hyades.statistical.ui");
//		String pluginID = "org.eclipse.hyades.statistical.ui";
//		String obj16 = "icons/full/obj16/";
//		
//		reg.put(IMG_COMMENT,	imageDescriptorFromPlugin(pluginID, obj16 + IMG_COMMENT));
	}
	
	
private String infopops_id = null;

	public String getInfopopsId() {
		if (infopops_id == null) {
			try {
				infopops_id = getBundle().getSymbolicName();
//				infopops_id = Platform.getPlugin("org.eclipse.hyades.statistical.ui").getDescriptor().getUniqueIdentifier();
				DBG.info("Infopops Plugin ID = "+infopops_id);
			} catch (Throwable t) {
				DBG.logVisibleError(t,getString("ERROR_NO_INFOPOPS_ID"),false);
				infopops_id = "org.eclipse.hyades.statistical.ui";
			}
		}
		return infopops_id;
	}
/*
	public void startup() throws CoreException {
		super.startup();
		EditorPlugin.DBG.info("calling StatisticalPackageImpl.init() to fix bugs");
		StatisticalPackageImpl.init();
	}  
*/
	/**
	 * Returns the shared instance.
	 */
	public static EditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getString(String key)
	{
		try {
			if (getDefault().resourceBundle == null) {
				getDefault().resourceBundle = Platform.getResourceBundle(getDefault().getBundle());
			}
			return getDefault().resourceBundle.getString(key);
		} catch (NullPointerException e) {
			return key;
		} catch (MissingResourceException e) {
			return key;
		}	
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
//	public ResourceBundle getResourceBundle() {
//		return resourceBundle;
//	}
	
	public static Shell getActiveWorkbenchShell() {
		return getActiveWorkbenchWindow().getShell();
	}
	public static IWorkbenchWindow getActiveWorkbenchWindow() {
		return getDefault().getWorkbench().getActiveWorkbenchWindow();
	}

}
