/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.statistical.ui.editor.internal;

import org.eclipse.emf.common.util.EList;
import org.eclipse.hyades.model.statistical.SDContiguousObservation;
import org.eclipse.hyades.model.statistical.SDDescriptor;
import org.eclipse.hyades.model.statistical.SDMemberDescriptor;
import org.eclipse.hyades.model.statistical.SDRepresentation;
import org.eclipse.hyades.model.statistical.SDSnapshotObservation;
import org.eclipse.hyades.model.statistical.SDTextObservation;
import org.eclipse.hyades.statistical.ui.EditorPlugin;
import org.eclipse.hyades.statistical.ui.ImageManager;
import org.eclipse.hyades.statistical.ui.variableloader.internal.SDModifiableVariableRepresentation;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.BasicGraphSource;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.Graph;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.ZoomSlider;
import org.eclipse.swt.widgets.TreeItem;

/**
 * This recursive StatCon operation generates the statistical model parts of the statcon configuration file 
 */
public class OpGenerateDescriptorConfig implements StatConModelOp {
	public static OpGenerateDescriptorConfig INSTANCE = new OpGenerateDescriptorConfig();

	static ImageManager img = EditorPlugin.img;

	public static void launch(StatisticalModelGraphViewer statcon, SDDescriptor item, SDDescriptor root, StringBuffer xml, int depth) {
		StatConOpState state = new StatConOpState(new State(depth,xml));
		statcon.applyStatConModelOp( new OpGenerateDescriptorConfig(), state, item, root);
	}

	public StatConOpState preapply(StatConOpState state, StatisticalModelGraphViewer statcon, SDDescriptor descriptor, SDDescriptor root_parent, boolean parents_checked, boolean parents_enabled) {
		State mystate = (State)state.state;
		StringBuffer xml = mystate.xml;
		int depth = mystate.depth;
		
		generatePreChildConfig(statcon,xml,descriptor,depth);
		
		return new StatConOpState(new State(depth+1,xml));
	}

	public void postapply(StatConOpState state, StatisticalModelGraphViewer statcon, SDDescriptor descriptor, SDDescriptor root_parent, boolean parents_checked, boolean parents_enabled) {
		State mystate = (State)state.state;
		StringBuffer xml = mystate.xml;
		int depth = mystate.depth;
		
		generatePostChildConfig(statcon,xml,descriptor,depth);
	}
	
	static class State {
		public State(int depth, StringBuffer xml) {
			this.depth = depth;
			this.xml = xml;
		}
		int depth = 0;
		StringBuffer xml;
	}


	private void generatePostChildConfig(StatisticalModelGraphViewer statcon, StringBuffer xml, SDDescriptor descriptor, int depth) {
		for (int t = 0; t < depth; t++) {
			xml.append("  ");	
		} 
		xml.append("</descriptor>\n");
	}
	
	private void generatePreChildConfig(StatisticalModelGraphViewer statcon, StringBuffer xml, SDDescriptor descriptor, int depth) {

//		TreeItem sd_item = (TreeItem)reverse_map.get(descriptor);
		TreeItem sd_item = (TreeItem)statcon.mapDescriptorToItem(descriptor);

		if (descriptor == null) return;
		if (sd_item == null) return;

		//
		// Append descriptor information
		//
		for (int t = 0; t < depth; t++) {
			xml.append("  ");	
		} 
		xml.append("<descriptor ");
		xml.append("name=\"").append(descriptor.getName()).append("\" ");
		xml.append("ticked=\"").append(sd_item.getChecked()).append("\" ");
		xml.append("expanded=\"").append(sd_item.getExpanded()).append("\" ");
		xml.append(">\n");
		
		//
		// Append descriptor graph informationm
		//
		if (descriptor instanceof SDMemberDescriptor) {
			SDMemberDescriptor mem_descriptor = (SDMemberDescriptor)descriptor;
			EList observations = mem_descriptor.getSnapshotObservation();
			
			for (int i = 0; i < observations.size(); i++) {
				Object o = observations.get(i);
				if (o instanceof SDContiguousObservation || o instanceof SDTextObservation) {
//					TreeItem obs_item = (TreeItem)reverse_map.get(o);
					TreeItem obs_item = (TreeItem)statcon.mapObservationToItem((SDSnapshotObservation)o);

//					BasicGraphSource source = (BasicGraphSource)source_map.get(obs_item);
					BasicGraphSource source = statcon.mapItemToGraphSource(obs_item);

					Graph graph = null;
					
//					Graph graph = (Graph)source_graph_map.get(source);
					if (source != null) {
						graph = statcon.mapSourceToGraph(source);
					}

					if (graph != null) {

						for (int t = 0; t < depth; t++) {
							xml.append("  ");	
						} 
						xml.append("<obs_graph ");
						xml.append("ticked=\"").append(obs_item.getChecked()).append("\" ");
						xml.append("obs_index=\"").append(i).append("\" ");
						xml.append("x_slider=\"").append(statcon.getGraphWindow().getXSlider(graph.getXSlider())).append("\" ");
						xml.append("y_slider=\"").append(statcon.getGraphWindow().getYSlider(graph.getYSlider())).append("\" ");
						xml.append("color=\"").append(XMLConfigUtil.colorToString(graph.getForeground())).append("\" ");
						xml.append("line_width=\"").append(graph.getLineWidth()).append("\" ");
						xml.append("line_style=\"").append(XMLConfigUtil.lineStyleToString(graph.getLineStyle())).append("\" ");
						xml.append("multiplier=\"").append(graph.getStaticScaling()).append("\" ");
						
						//3.1 additions
						xml.append("plotting_type=\"").append(XMLConfigUtil.generatePlottingType(graph.getPlottingType())).append("\" ");
						xml.append("plotting_period=\"").append(XMLConfigUtil.generatePlottingPeriod(graph.getPlottingPeriodType())).append("\" ");
						xml.append("nodata_behaviour=\"").append(XMLConfigUtil.generateNoDataBehaviour(graph.getNoDataBehaviour())).append("\" ");
						xml.append("plotting_period_value=\"").append(graph.getPlottingPeriodValue()).append("\" ");
						xml.append("xoffset=\"").append(graph.getXOffset()).append("\" ");
						xml.append("yoffset=\"").append(graph.getYOffset()).append("\" ");
						
						xml.append("/>\n");
					
					} else if (obs_item != null) {
					
						//this graph was never shown and it therefore has no settings, so we just make a note of whether the tree item was checked
						for (int t = 0; t < depth; t++) {
							xml.append("  ");	
						} 

						xml.append("<obs_graph ");
						xml.append("ticked=\"").append(obs_item.getChecked()).append("\" ");
						xml.append("obs_index=\"").append(i).append("\" ");

						xml.append("/>\n");
						
					}
				}
				
			}	
		}
		
		//
		// Append descriptor control information
		//
		if (descriptor instanceof SDMemberDescriptor) {
			SDMemberDescriptor mem_descriptor = (SDMemberDescriptor)descriptor;
			SDRepresentation rep = mem_descriptor.getRepresentation();
			
			if (rep instanceof SDModifiableVariableRepresentation) {
//				TreeItem rep_item = (TreeItem)reverse_map.get(rep);
				TreeItem rep_item = statcon.mapRepresentationToItem(rep);

//				ZoomSlider slider = (ZoomSlider)bar_slider_map.get(rep_item);
				ZoomSlider slider = statcon.mapItemToSlider(rep_item);

				for (int t = 0; t < depth; t++) {
					xml.append("  ");	
				} 
				
				if (slider != null) {
					xml.append("<control ");
					xml.append("ticked=\"").append(rep_item.getChecked()).append("\" ");
					xml.append("y_slider=\"").append(statcon.getGraphWindow().getYSlider(slider)).append("\" ");
					xml.append("/>\n");
				} else {
					xml.append("<control ");
					xml.append("ticked=\"").append(rep_item.getChecked()).append("\" ");
					xml.append("y_slider=\"0\" ");
					xml.append("/>\n");
				}
				
			}
			
		}
/*		
		
		//
		// Append descriptor children
		//
		EList list = descriptor.getChildren();
		for (int i = 0; i < list.size(); i++) {
			SDDescriptor child = (SDDescriptor)list.get(i);
			generateConfig(xml,child,depth+1);	
		}
*/				
	}

}