/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.statistical.ui.editor.internal;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.hyades.model.statistical.SDContiguousObservation;
import org.eclipse.hyades.model.statistical.SDDescriptor;
import org.eclipse.hyades.model.statistical.SDDiscreteObservation;
import org.eclipse.hyades.model.statistical.SDMemberDescriptor;
import org.eclipse.hyades.model.statistical.SDRepresentation;
import org.eclipse.hyades.model.statistical.SDSnapshotObservation;
import org.eclipse.hyades.model.statistical.SDTextObservation;
import org.eclipse.hyades.models.cbe.CBECommonBaseEvent;
import org.eclipse.hyades.models.cbe.CBEComponentIdentification;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.util.HierarchyResourceSetImpl;
import org.eclipse.hyades.statistical.ui.EditorPlugin;
import org.eclipse.hyades.statistical.ui.ImageManager;
import org.eclipse.hyades.statistical.ui.variableloader.internal.SDMeasuredVariableRepresentation;
import org.eclipse.hyades.statistical.ui.variableloader.internal.SDModifiableVariableRepresentation;
import org.eclipse.hyades.statistical.ui.widgets.alert.internal.Alert;
import org.eclipse.hyades.statistical.ui.widgets.alert.internal.AlertListener;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.BasicGraphSource;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.Dirtiable;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.EnumerationGraph;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.EnumerationGraphSource;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.Graph;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.GraphSource;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.HistogramGraph;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.LineGraph;
import org.eclipse.hyades.statistical.ui.widgets.grapher.internal.TestGraphSource;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.TimeZoomSlider;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.ZoomControlBar;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.ZoomImageBar;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.ZoomSlider;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.eclipse.ui.part.ResourceTransfer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Creates a viewer/editor of a statistical model and its aggregates
 * the aim is to be able to see some kind of tree of the statistical model
 * and then pick any set of observations you like to be graphed
 * 
 */
public class StatisticalModelGraphViewer implements AlertListener, DisposeListener, SelectionListener, DropTargetListener, MouseListener, StatConInterface, GraphWindowListener {

private static final boolean DEBUG_LOADING = true;
public static final int RESERVED_NODES = 1;

protected List disposables = Collections.synchronizedList(new ArrayList());

protected Map module_refs = Collections.synchronizedMap(new HashMap());
protected List modules = Collections.synchronizedList(new ArrayList());
protected List stat_img_providers = Collections.synchronizedList(new ArrayList());
protected List alerts = Collections.synchronizedList(new ArrayList());

protected int next = 0;
protected int[] colours = new int[]{
	SWT.COLOR_RED,	
	SWT.COLOR_GREEN,	
	SWT.COLOR_BLUE,	
	SWT.COLOR_YELLOW,	
//	SWT.COLOR_BLACK,	//fails in Linux
	SWT.COLOR_CYAN,	
	SWT.COLOR_MAGENTA,	
};

protected TabFolder tabfolder;
protected SashForm sashfolder;
protected Composite leftsash;
protected Composite rightsash;

protected GraphWindow graph_window;

protected Tree chooser_tree;

protected Menu chooser_menu;
protected ArrayList chooser_disposables;
protected MenuItem chooser_add;

//SDDescriptor selected_model;

protected ImageManager img;

protected Map reverse_map = Collections.synchronizedMap(new HashMap());			//key=SDDescriptor, value=TreeItem
	
protected Map trcagent_map = Collections.synchronizedMap(new HashMap());			//key=TreeItem, value=TRCAgent
protected Map msglist_map = Collections.synchronizedMap(new HashMap());			//key=TreeItem, value=EList (TRCAgent.getDefaultEvents())
protected Map sddescriptor_map = Collections.synchronizedMap(new HashMap());		//key=TreeItem, value=SDDescriptor
protected Map counter_map = Collections.synchronizedMap(new HashMap());			//key=TreeItem, value=SDSnapshotObservation
protected Map modifiable_map = Collections.synchronizedMap(new HashMap());		//key=TreeItem, value=SDRepresentation

protected Map rgraph_map = Collections.synchronizedMap(new HashMap());			//key=Graph, value=TreeItem
protected Map graph_map = Collections.synchronizedMap(new HashMap());			//key=TreeItem, value=Graph
protected Map bar_map = Collections.synchronizedMap(new HashMap());				//key=TreeItem, value=ModifiableVariableBar
protected Map bar_slider_map = Collections.synchronizedMap(new HashMap());		//key=TreeItem, value=ZoomSlider
protected Map source_map = Collections.synchronizedMap(new HashMap());			//key=TreeItem, value=GraphSource
protected Map source_graph_map = Collections.synchronizedMap(new HashMap());		//key=GraphSource, value=Graph

protected Dirtiable dirtiable;

protected java.util.List agents = Collections.synchronizedList(new ArrayList());

protected java.util.List dynamic_actions = Collections.synchronizedList(new ArrayList());

protected java.util.List click_listeners = Collections.synchronizedList(new ArrayList());
protected java.util.List global_actions = Collections.synchronizedList(new ArrayList());
protected java.util.List drop_listeners = Collections.synchronizedList(new ArrayList());
protected java.util.List root_nodes = Collections.synchronizedList(new ArrayList());

protected java.util.List model_change_listeners = Collections.synchronizedList(new ArrayList());
protected java.util.List agent_change_listeners = Collections.synchronizedList(new ArrayList());
protected java.util.List node_change_listeners = Collections.synchronizedList(new ArrayList());

protected URI current_project;

	public void setCurrentProjectURI(URI uri) {
		current_project = uri;	
	}
	
	public URI getCurrentProjectURI() {
		return current_project;	
	}
	
	public void showTab(int i) {
		graph_window.showTab(i);	
	}

	public StatisticalModelGraphViewer(Composite parent, int style, boolean tabfolder) {		
		EditorPlugin.DBG.info("constructor");
		initAsSash(parent);
		initCommonActions();
	}
	
	public StatisticalModelGraphViewer(Composite parent, int style, boolean tabfolder, Dirtiable d, URI current_project) {		
		EditorPlugin.DBG.info("constructor");
		dirtiable = d;

		this.current_project = current_project;

		initAsSash(parent);
		initCommonActions();

		loadModules();

	}

	public void addClickListener(ClickListener listener) {
		click_listeners.add(listener);
	}
	public void removeClickListener(ClickListener listener) {
		click_listeners.remove(listener);
	}
	
	public void addModuleNodeChangeListener(ModuleNodeChangeListener listener) {
		node_change_listeners.add(listener);	
	}

	public void removeModuleNodeChangeListener(ModuleNodeChangeListener listener) {
		node_change_listeners.remove(listener);	
	}
	
	public void addModelChangeListener(ModelChangeListener listener) {
		model_change_listeners.add(listener);	
	}

	public void removeModelChangeListener(ModelChangeListener listener) {
		model_change_listeners.remove(listener);	
	}

	public void addAgentChangeListener(AgentChangeListener listener) {
		agent_change_listeners.add(listener);	
	}

	public void removeAgentChangeListener(AgentChangeListener listener) {
		agent_change_listeners.remove(listener);	
	}

	public String[] getModuleRefs() {
		String[] refs = new String[modules.size()];
		for (int i = 0; i < modules.size(); i++) {
			refs[i] = ((StatConModule)modules.get(i)).getModuleRef();
		}
		return refs;
	}
	
	public StatConModule getModuleByRef(String ref) {
		return (StatConModule)module_refs.get(ref);
	}
	
	public void unloadModules() {
		EditorPlugin.DBG.info("unloading modules");
		
		for (int i = 0; i < modules.size(); i++) {
			try {
				StatConModule module = (StatConModule)modules.get(i);
				EditorPlugin.DBG.info("unloading module "+i);
				module.unload();	
			} catch (Throwable t) {
				EditorPlugin.DBG.error("failed to unload module",t);
			}
		}
		
		modules.clear();
		module_refs.clear();
	}
	
	public ZoomSlider getSelectedSlider() {
		return graph_window.getSelectedSlider();
	}
	
	public double getSelectedSliderValue() {
		return graph_window.getSelectedSliderValue();
	}

	public void addSliderAction(String id, String name, Image image, Runnable runnable, boolean horizontal) {
		graph_window.addSliderAction(id,name,image,runnable,horizontal);
	}

	public void removeSliderAction(String id) {
		graph_window.removeSliderAction(id);
	}

	public void loadModules() {
		EditorPlugin.DBG.info("loading modules");

		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EditorPlugin.PACKAGE_STATCON_MODULE);
		IExtension[] extensions = point.getExtensions();
//		IPluginRegistry registry = Platform.getPluginRegistry();
//		IExtensionPoint point = registry.getExtensionPoint(EditorPlugin.PACKAGE_STATCON_MODULE);
//		IExtension[] extensions = point.getExtensions();

		EditorPlugin.DBG.info("found "+extensions.length+" modules");

		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] configs = extensions[i].getConfigurationElements();
//			IPluginDescriptor pdescriptor = extensions[i].getDeclaringPluginDescriptor();
			
			for (int z = 0; z < configs.length; z++) {
				String clazz = configs[z].getAttribute("class");

				StatConModule module = null;

				try {			
					EditorPlugin.DBG.info("loading module "+i);
//					ClassLoader cloader = pdescriptor.getPluginClassLoader();	
//					Class c = cloader.loadClass(clazz);
					module = (StatConModule)configs[z].createExecutableExtension("class");
//					module = (StatConModule)c.newInstance();
					module.load(this);
					module_refs.put(module.getModuleRef(),module);
					modules.add(module);
					EditorPlugin.DBG.info("loaded OK");
				} catch (Throwable e) {
					try {
						modules.remove(module);
					} catch (Throwable x) {}
					try {
						module_refs.remove(module);
					} catch (Throwable x) {}
					EditorPlugin.DBG.logVisibleError(e,EditorPlugin.getString("ERROR_LOAD_MODULE")+" \""+clazz+"\" ",false);
					EditorPlugin.DBG.error("failed to load module",e);
				}
			}	
		}

		EditorPlugin.DBG.info("finished loading modules");

		EditorPlugin.DBG.info("loading statistical image providers");
		
		registry = Platform.getExtensionRegistry();
		point = registry.getExtensionPoint(EditorPlugin.PACKAGE_STATCON_STAT_IMG_PROVIDER);
		extensions = point.getExtensions();
		
		EditorPlugin.DBG.info("found "+extensions.length+" statistical image providers");

		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] configs = extensions[i].getConfigurationElements();
			
			for (int z = 0; z < configs.length; z++) {
				String clazz = configs[z].getAttribute("class");

				StatConStatisticalImageProvider imgprovider = null;

				try {			
					EditorPlugin.DBG.info("loading statistical image provider "+i);
					imgprovider = (StatConStatisticalImageProvider)configs[z].createExecutableExtension("class");
					stat_img_providers.add(imgprovider);
				} catch (Throwable e) {
					EditorPlugin.DBG.logVisibleError(e,EditorPlugin.getString("ERROR_LOAD_STAT_IMGPROVIDER")+" \""+clazz+"\" ",false);
					EditorPlugin.DBG.error("failed to load statistical image provider",e);
				}
			}
		}

		EditorPlugin.DBG.info("finished loading statistical image providers");
	}		

	private DynamicAction findDynamicAction(String id, java.util.List actions) {
		for (int i = 0; i < actions.size(); i++) {
			DynamicAction action = (DynamicAction)actions.get(i);
			if (action.id.equals(id)) {
				return action;
			}	
		}
		return null;
	}
	
	private BasicAction findAction(String id, java.util.List actions) {
		for (int i = 0; i < actions.size(); i++) {
			BasicAction action = (BasicAction)actions.get(i);
			if (action.id.equals(id)) {
				return action;
			}	
		}
		return null;
	}
	
	private RootNode getRootNodeName(String name) {
		for (int i = 0; i < root_nodes.size(); i++) {
			RootNode node = (RootNode)root_nodes.get(i);
			if (node.name.equals(name)) {
				return node;
			}	
		}
		return null;
	}
	private RootNode getRootNodeID(String id) {
		for (int i = 0; i < root_nodes.size(); i++) {
			RootNode node = (RootNode)root_nodes.get(i);
			if (node.id.equals(id)) {
				return node;
			}	
		}
		return null;
	}

	private StatAgent getStatAgent(TRCAgent agent) {
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			if (node.agent == agent) {
				return node;
			}	
		}
		return null;
	}

	private StatAgent getStatAgent(EList events) {
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			if (node.agent.getDefaultEvents() == events) {
				return node;
			}	
		}
		return null;
	}

	public TRCAgent[] getAgents() {
		TRCAgent[] tmp = new TRCAgent[agents.size()];
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			tmp[i] = node.agent;
		}
		return tmp;
	}
	
	public TRCAgent getAgentForURI(URI uri) {
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			if (node.uri != null) {
				if (node.uri.toString().equals(uri.toString())) {
					return node.agent;
				}
			}
		}
		return null;
	}
	
	public void addGlobalAction(String id, String name, Image image, Runnable runnable) {
		BasicAction action = findAction(id,global_actions);
		if (action == null) {
			action = new BasicAction();
			global_actions.add(action);
		}
		action.id = id;
		action.name = name;
		action.image = image;
		action.runnable = runnable;
		action.menuitem = new MenuItem(chooser_menu,0);
		action.menuitem.setText(name);
		action.menuitem.setImage(image);
		action.menuitem.addSelectionListener(action);
	}

	public void removeGlobalAction(String id) {
		BasicAction action = findAction(id,global_actions);
		if (action != null) {
			action.dispose();
			global_actions.remove(action);
		}	
	}

	public void addDynamicAction(String group, String id, String name, DynamicAction action) {
		if (findDynamicAction(id,dynamic_actions) == null) {
			action.id = id;
			action.name = name;
			action.group = group;
			dynamic_actions.add(action);
		}
	}
	public void removeDynamicAction(String id) {
		DynamicAction tmp = findDynamicAction(id,dynamic_actions);
		if (tmp != null) dynamic_actions.remove(tmp);
	}
	
	public void addDropTargetListener(String extension, StatConDropListener listener) {
		DropListener drop = new DropListener();
		drop.extension = extension;
		drop.listener = listener;
		
		drop_listeners.add(drop);
	}
	
	public void removeDropTargetListener(String extension) {
		for (int i = 0; i < drop_listeners.size(); i++) {
			DropListener drop = (DropListener)drop_listeners.get(i);
			if (drop.extension.equals(extension)) {
				drop_listeners.remove(drop);
			}	
		}
	}
	public void addModuleNode(String name, String id, Image image) throws EntityExistsException {
		if (getRootNodeID(id) != null) {
			throw new EntityExistsException(EditorPlugin.getString("ROOT_NODE_EXISTS")+" "+name+" "+EditorPlugin.getString("ROOT_NODE_EXISTS_2"));	
		}
		
		RootNode node = new RootNode();
		node.name = name;
		node.image = image;
		node.id = id;

		node.item = new TreeItem(chooser_tree,0,0);
		node.item.setText(name);
		node.item.setImage(image);
		node.item.setGrayed(true);

		root_nodes.add(node);
		
		for (int i = 0; i < node_change_listeners.size(); i++) {
			ModuleNodeChangeListener listener = (ModuleNodeChangeListener)node_change_listeners.get(i);	
			listener.nodeAdded(id);
		}
	}

	public void addModuleNode(String name, String id, String parent_id, Image image) throws EntityExistsException, NullPointerException {
		if (getRootNodeID(id) != null) {
			throw new EntityExistsException(EditorPlugin.getString("ROOT_NODE_EXISTS")+" "+name+" "+EditorPlugin.getString("ROOT_NODE_EXISTS_2"));	
		}

		RootNode parent = getRootNodeID(parent_id);
		if (parent == null) throw new NullPointerException("");
		
		RootNode node = new RootNode();
		node.name = name;
		node.image = image;
		node.id = id;
		node.parent_id = parent_id;

		node.item = new TreeItem(parent.item,0,0);
		node.item.setText(name);
		node.item.setImage(image);
		node.item.setGrayed(true);

		parent.node_children.add(node);

		root_nodes.add(node);

		for (int i = 0; i < node_change_listeners.size(); i++) {
			ModuleNodeChangeListener listener = (ModuleNodeChangeListener)node_change_listeners.get(i);	
			listener.nodeAdded(id);
		}
	}
	
	public void removeModuleNode(String id) {
		RootNode node = getRootNodeID(id);
		if (node != null) {
			
			if (node.parent_id != null) {
				RootNode parent = getRootNodeID(node.parent_id);
				parent.node_children.remove(node);
			}

			for (int i = 0; i < node.node_children.size(); i++) {
				RootNode child = (RootNode)node.node_children.get(i);
				removeModuleNode(child.id);	
			}
			
			node.dispose();
			root_nodes.remove(node);
		}	

		for (int i = 0; i < node_change_listeners.size(); i++) {
			ModuleNodeChangeListener listener = (ModuleNodeChangeListener)node_change_listeners.get(i);	
			listener.nodeRemoved(id);
		}
	}
	
	public void addModuleNodeAction(String node_id, String action_id, String action_name, Image action_image, Runnable runnable) {
		RootNode node = getRootNodeID(node_id);
		if (node != null) {
			BasicAction action = findAction(action_id,node.actions); 
			if (action == null) {
				action = new BasicAction();
				node.actions.add(action);	
			}

			action.id = action_id;
			action.name = action_name;
			action.image = action_image;
			action.runnable = runnable;
	
		}	
	}

	public void removeModuleNodeAction(String node_id, String action_id) {
		RootNode node = getRootNodeID(node_id);
		if (node != null) {
			for (int i = 0; i < node.actions.size(); i++) {
				BasicAction action = (BasicAction) node.actions.get(i);
				if (action.id.equals(action_id)) {
					node.actions.remove(i);
					break;
				}
			}
		}
	}
	
	public SDDescriptor[] addAgent(IFile ifile) throws IOException {
		try {
			return addAgent(ifile.getProjectRelativePath());
		} catch (Exception e) {
			return addAgent(ifile.getFullPath());
		}
	}
	
	private Resource createHierarchyResource(URI uri) {
		try {
			ResourceSet resourceSet = HierarchyResourceSetImpl.getInstance();
			Resource resource = resourceSet.getResource(uri,false);
			if (resource == null) return resourceSet.createResource(uri);
			return resource;		
		} catch (Exception e) {
			EditorPlugin.DBG.warning("Problem creating Hierarchy resource",e);
			return null;
		}
	}
	private Resource createResource(URI uri) {

		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(uri,false);
		if (resource == null) return resourceSet.createResource(uri);
		return resource;		
	}
	
	public SDDescriptor[] addAgent(IPath ipath) throws IOException {

		EditorPlugin.DBG.info("add trace agent");

		URI fileURI = URI.createPlatformResourceURI(ipath.toString());

		EditorPlugin.DBG.info("agent URI:"+fileURI);

		if (fileURI == null) throw new IOException(EditorPlugin.getString("NULL_URI_WHILE_LOADING")+" "+ipath);

		Resource resource = createResource(fileURI);
//		Resource resource = createHierarchyResource(fileURI);
		if (resource == null) {
			resource = createResource(fileURI);
		}
		if (resource == null) throw new IOException(EditorPlugin.getString("NULL_RESOURCE_WHILE_LOADING")+" "+ipath);

		if (resource instanceof XMLResource) {
			EditorPlugin.DBG.info("found XML resource for agent");
			XMLResource xresource = (XMLResource)resource;
			
			String msg = "";
			
			try {
				//try to load as a non-zip file
				EditorPlugin.DBG.info("trying to load resource as an unzipped file");
				xresource.setUseZip(false);
				xresource.load(Collections.EMPTY_MAP);
				EditorPlugin.DBG.info("loaded resource OK as an unzipped file");
			} catch (Throwable e) {

				EditorPlugin.DBG.info("load failed as an unzipped file");

				msg += e.toString()+" / ";

				resource = createHierarchyResource(fileURI);
				if (resource == null) {
					resource = createResource(fileURI);
				}
				xresource = (XMLResource)resource;
				
				try {
					//try to load as a zip file
					EditorPlugin.DBG.info("trying to load resource as a zipped file");
					xresource.setUseZip(true);
					xresource.load(Collections.EMPTY_MAP);
					EditorPlugin.DBG.info("loaded resource OK as a zipped file");
				} catch (Throwable t) {
					msg += t.toString();
					throw new IOException(msg); 
				}
			
			}
			
		} else {
			EditorPlugin.DBG.info("found a non-XML resource for agent - trying to load anyway");
			if (!resource.isLoaded()) resource.load(Collections.EMPTY_MAP);
		}

		EList list = resource.getContents();
		EditorPlugin.DBG.info("got contents of agent resource (size "+list.size()+")");
		
		return addAgentFromTree(list,fileURI);
	}
	
	private SDDescriptor[] addAgentFromTree(EList list, URI fileURI) {
		
		EditorPlugin.DBG.info("Searching Resource Contents ("+list.size()+")");
		
		//check the list for a TRCAgent, if we find one add it
		for (int k = 0; k < list.size(); k++) {
			Object o = list.get(k);
			EditorPlugin.DBG.info("resource content "+k+" = "+o.getClass());

			if (o instanceof TRCAgentProxy) {
				TRCAgentProxy proxy = (TRCAgentProxy)o;
				return addAgent(proxy.getAgent(),fileURI);
			}

			if (o instanceof TRCAgent) {
				return addAgent((TRCAgent)o,fileURI);
			}
		}
		//no TRCAgent found - recursively search on all its children
		SDDescriptor[] descriptors = null;
		for (int k = 0; k < list.size(); k++) {
			EObject o = (EObject)list.get(k);
			if (o instanceof TRCProcessProxy) {
				EditorPlugin.DBG.info("resource subcontent ProcessProxy");
				descriptors = addAgentFromTree(((TRCProcessProxy)o).getAgentProxies(),fileURI);
			} else if (o instanceof TRCNode) {
				EditorPlugin.DBG.info("resource subcontent Node");
				descriptors = addAgentFromTree(((TRCNode)o).getProcessProxies(),fileURI);
			} else if (o instanceof TRCMonitor) {
				EditorPlugin.DBG.info("resource subcontent Monitor");
				descriptors = addAgentFromTree(((TRCMonitor)o).getNodes(),fileURI);
			} else {
			EditorPlugin.DBG.info("resource subcontent "+k+" = "+o.getClass());
				descriptors = addAgentFromTree(o.eContents(),fileURI);
			}
			if (descriptors != null) {
				return descriptors;
			}
		}
		return null;
	}

	public SDDescriptor[] addAgent(TRCAgent agent) {
		return addAgent(agent,null);
	}

	SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
	private StringBuffer cbeToString(CBECommonBaseEvent e, StringBuffer sb) {
		
		//times are in milliseconds
		Date create = new Date((long)(e.getCreationTime()/1000));
		
		sdf.format(create,sb,new FieldPosition(0));
		sb.append(" ");
	
		short sev = e.getSeverity();
		if (sev <= 10) {
			sb.append("("+EditorPlugin.getString("CBE_INFO")+")");
		} else if (sev <= 30) {
			sb.append("("+EditorPlugin.getString("CBE_WARNING")+")");
		} else if (sev <= 40) {
			sb.append("("+EditorPlugin.getString("CBE_ERROR")+")");
		} else if (sev <= 60) {
			sb.append("("+EditorPlugin.getString("CBE_CRITICAL")+")");
		}

		sb.append(" ");

		if (e.getRepeatCount() > 0) {
			sb.append("(x");				
			sb.append(e.getRepeatCount());
			sb.append(") ");				
		}

//		sb.append(e.getAgent().getName());
//
//		sb.append(" / ");

		sb.append(e.getSourceComponentId().getComponent());
		
		if (e.getSourceComponentId().getSubComponent() != null) {
			sb.append(" / ");
			sb.append(e.getSourceComponentId().getSubComponent());
		}

		sb.append(" - ");
		sb.append(e.getMsg());

		sb.append("\n");

		return sb;
	}
	
	private boolean filterCBE(CBECommonBaseEvent e) {
		
		TRCAgent agent = e.getAgent();
		
		if (agent == null) return false;
		
		StatAgent stagent = getStatAgent(agent);
		
		if (stagent == null) return false;
		if (stagent.msg_info_item == null) return false;
		
		//check general message filtering
		if (!stagent.msg_item.getChecked()) return false;
		
		//check severity filtering
		
		int severity = e.getSeverity();
		if (severity <= 10) {
			if (!stagent.msg_info_item.getChecked()) return false; 
		} else if (severity <= 30) {
			if (!stagent.msg_warn_item.getChecked()) return false; 
		} else {
			if (!stagent.msg_error_item.getChecked()) return false; 
		}
		
		//check location filtering
		CBEComponentIdentification sid = e.getSourceComponentId();
		String component = sid.getComponent();
		String subcomponent = sid.getSubComponent();
		
		TreeItem citem = (TreeItem)stagent.msg_locations.get(component);
		if (citem == null) {
			citem = new TreeItem(stagent.msg_loc_item,0);
			citem.setImage(img.getImage(ImageManager.IMG_SMODEL_CBESOURCE));
			citem.setText(component);
			citem.setChecked(true);
			stagent.msg_locations.put(component,citem);
		}
		if (!citem.getChecked()) return false;
		
		if (subcomponent != null) {
			String key = component+"<%$%~%>"+subcomponent;
			TreeItem sitem = (TreeItem)stagent.msg_locations.get(key);
			if (sitem == null) {
				sitem = new TreeItem(citem,0);
				sitem.setImage(img.getImage(ImageManager.IMG_SMODEL_CBESOURCE));
				sitem.setText(subcomponent);
				sitem.setChecked(true);
				stagent.msg_locations.put(key,sitem);
			}
			if (!sitem.getChecked()) return false;
		}
		
		TreeItem[] items = stagent.msg_loc_item.getItems();

		return true;
	}
	
	private int appendCBEs(java.util.List events, int index) {
		StringBuffer sb = new StringBuffer();
		for (int i = index; i < events.size(); i++) {
			CBECommonBaseEvent e = (CBECommonBaseEvent)events.get(i);
			if (filterCBE(e)) {
				cbeToString(e,sb);
			}
		}
		graph_window.appendCBEText(sb.toString());
		return events.size();
	}

	private int setCBEs(java.util.List events, int index) {
		StringBuffer sb = new StringBuffer();
		for (int i = index; i < events.size(); i++) {
			CBECommonBaseEvent e = (CBECommonBaseEvent)events.get(i);
			if (filterCBE(e)) {
				cbeToString(e,sb);
			}
		}
		graph_window.setCBEText(sb.toString());
		return events.size();
	}

	private void sortCBEs(java.util.List events) {
		Collections.sort(events,new CBEComparator());	
	}

	private void appendCBEs(StatAgent agent) {
		if (agent.show_messages) {
			//append all unappended messages from this agent
			agent.msgindex = appendCBEs(agent.agent.getDefaultEvents(),agent.msgindex);
		}	
	}

	private void appendAllCBEs() {
		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);
			//append all unappended messages from this agent
			agent.msgindex = appendCBEs(agent.agent.getDefaultEvents(),agent.msgindex);
		}	
	}
	
	private void redoAllCBEs() {
		ArrayList all_events = new ArrayList();
		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);

			if (agent.show_messages) {
				agent.msgindex = (agent.agent.getDefaultEvents().size());
				all_events.addAll(agent.agent.getDefaultEvents());
			}

		}
		sortCBEs(all_events);
		setCBEs(all_events,0);
	}

	public void updateCBELog() {
		appendAllCBEs();
	}

	public void reloadCBELog() {
		redoAllCBEs();
	}

	public SDDescriptor[] updateAgent(TRCAgent agent) {

		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			EditorPlugin.DBG.info("agent "+i+" = "+node.agent.getName());
		}

		StatAgent statagent = getStatAgent(agent);
		
		if (statagent == null) {
			EditorPlugin.DBG.info("update requested on non-existent agent "+agent.getName());
			return null;
		}

		EList direct_children = agent.eContents();

		EditorPlugin.DBG.info("checking agent's "+direct_children.size()+" children for Statistical Models");

		ArrayList added = new ArrayList();
	
		for (int z = 0; z < direct_children.size(); z++) {
			Object co = direct_children.get(z);

			EditorPlugin.DBG.info("agent child "+z+" = "+co.getClass());
		
			if (co instanceof SDDescriptor) {
				EditorPlugin.DBG.info("adding agent SDDescriptor (statistical model)");

				//if model root already there, update it, otherwise add it
				if (statagent.model_roots.contains(co)) {
//					updateModel((SDDescriptor)co,statagent);
					updateModel((SDDescriptor)co);
					added.add(co);
				} else { //new model - add it
					addModel((SDDescriptor)co,statagent);
					added.add(co);
				}
			}
		}
		
		TreeItem agent_item = statagent.msg_item.getParentItem();
		TreeItem[] items = agent_item.getItems();
		for (int i = 0; i < items.length; i++) {
			if (items[i] != statagent.msg_item) {
				if (mapItemToDescriptor(items[i]) == null) {
					if (!items[i].isDisposed()) {
						items[i].dispose();
					}
				}	
			}	
		}
		
		SDDescriptor[] des = new SDDescriptor[added.size()];
		for (int i = 0; i < des.length; i++) {
			des[i] = (SDDescriptor)added.get(i);	
		}

		updateTreeItems();

		try {
			chooser_tree.redraw();
		} catch (Exception e) {}
		
		return des;
	}		

	public SDDescriptor[] addAgent(TRCAgent agent, URI fileURI) {

		if (agent == null) {
			EditorPlugin.DBG.logVisibleError(new Exception("Null Agent"),EditorPlugin.getString("ERROR_NULL_AGENT")+" "+fileURI,false);
			return null;
		}
		if (agent.getName() == null) {
			EditorPlugin.DBG.error(EditorPlugin.getString("ERROR_NULL_NAME")+" "+fileURI,new Exception("Null Agent Name"));
//			return null;
		}

		//
		// This bit is just for our info
		//
		EditorPlugin.DBG.info("add valid agent "+agent.getName()+" / "+fileURI);
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			EditorPlugin.DBG.info("agent "+i+" = "+node.agent.getName());
		}
		
		if (fileURI != null) {
			//dont add models that reference an already loaded file
			//(dont load same model twice)
			for (int i = 0; i < agents.size(); i++) {
				StatAgent node = (StatAgent)agents.get(i);
				URI uri = node.uri;
				if (uri != null) {
					if (uri.equals(fileURI)) {
						EditorPlugin.DBG.info("URI matched to an existing agent - returning without adding");
						return null;	
					}	
				}	
			}
		}

		EditorPlugin.DBG.info("creating new agent");

		//
		// Create the agent object and add it
		//		
		StatAgent statagent = getStatAgent(agent);
		if (statagent == null) {
			statagent = new StatAgent();
			agents.add(statagent);
		}
		statagent.agent = agent;
		statagent.uri = fileURI;
		
		TreeItem item = new TreeItem(chooser_tree,0);	
		if (agent.getName() == null) {
			if (agent.getAgentProxy().getName() != null) {
				item.setText(agent.getAgentProxy().getName());
			} else {
				item.setText(agent.getType()+" / "+agent.getAgentProxy().getProcessProxy().getName()+" / "+agent.getRuntimeId());
			}
		} else {
			item.setText(agent.getName());
		}
		item.setChecked(true);
		item.setGrayed(false);
		item.setImage(img.getImage(ImageManager.IMG_SMODEL_TRCAGENT));

		trcagent_map.put(item,agent);
		reverse_map.put(agent,item);

		//
		// Add a remove action
		//
//		addAgentAction(statagent.agent,"AGENT_REMOVE",EditorPlugin.getString("AGENT_REMOVE"),img.getImage(ImageManager.IMG_SMODEL_REMOVE),new AgentRemoveAction(statagent.agent));
//		this.addModelAction(statagent.agent,"MODEL_DELETE","Delete This Subtree",img.getImage(ImageManager.IMG_SMODEL_REMOVE),new TreeRemoveAction());

		//
		// Add all the messages from this agent
		//
		EList events = agent.getDefaultEvents();
		
		TreeItem msg = new TreeItem(item,0);	
		msg.setImage(img.getImage(ImageManager.IMG_SMODEL_CBEMESSAGE));
		msg.setText(EditorPlugin.getString("MESSAGES"));
		msg.setChecked(true);
		msg.setGrayed(false);

		TreeItem msg_severity = new TreeItem(msg,0);	
		msg_severity.setImage(img.getImage(ImageManager.IMG_SMODEL_CBESEVERITY));
		msg_severity.setText(EditorPlugin.getString("SEVERITY"));
		msg_severity.setChecked(true);
		msg_severity.setGrayed(true);

		TreeItem msg_location = new TreeItem(msg,0);	
		msg_location.setImage(img.getImage(ImageManager.IMG_SMODEL_CBESOURCE));
		msg_location.setText(EditorPlugin.getString("SOURCE_LOCATIONS"));
		msg_location.setChecked(true);
		msg_location.setGrayed(true);
		
		TreeItem msg_info = new TreeItem(msg_severity,0);
		msg_info.setImage(img.getImage(ImageManager.IMG_SMODEL_CBEINFO));
		msg_info.setText(EditorPlugin.getString("INFO"));
		msg_info.setChecked(true);
		msg_info.setGrayed(false);

		TreeItem msg_warning = new TreeItem(msg_severity,0);
		msg_warning.setImage(img.getImage(ImageManager.IMG_SMODEL_CBEWARNING));
		msg_warning.setText(EditorPlugin.getString("WARNING"));
		msg_warning.setChecked(true);
		msg_warning.setGrayed(false);

		TreeItem msg_errors = new TreeItem(msg_severity,0);
		msg_errors.setImage(img.getImage(ImageManager.IMG_SMODEL_CBEERROR));
		msg_errors.setText(EditorPlugin.getString("ERROR"));
		msg_errors.setChecked(true);
		msg_errors.setGrayed(false);

		msglist_map.put(msg,events);
		reverse_map.put(events,msg);
		
		statagent.msg_item = msg;
		statagent.msg_info_item = msg_info;
		statagent.msg_warn_item = msg_warning;
		statagent.msg_error_item = msg_errors;
		statagent.msg_loc_item = msg_location;

		//
		// Add all CBE messages
		//	
//		appendCBEs(statagent);
		EditorPlugin.DBG.info("interleaving all agent CBE messages");
		redoAllCBEs();
		
		//
		// Add all statistical models under this agent
		//
		ArrayList added = new ArrayList();

		EList direct_children = agent.eContents();

		EditorPlugin.DBG.info("checking agent's "+direct_children.size()+" children for Statistical Models");
	
		for (int z = 0; z < direct_children.size(); z++) {
			Object co = direct_children.get(z);

			EditorPlugin.DBG.info("agent child "+z+" = "+co.getClass());
		
			if (co instanceof SDDescriptor) {
				EditorPlugin.DBG.info("adding agent SDDescriptor (statistical model)");

				if (!statagent.model_roots.contains(co)) {
					addModel((SDDescriptor)co,statagent);
					added.add(co);
				}
			}
		}

		SDDescriptor[] des = new SDDescriptor[added.size()];
		for (int i = 0; i < des.length; i++) {
			des[i] = (SDDescriptor)added.get(i);	
		}

		//
		// This bit is just for our information
		//
		EditorPlugin.DBG.info("agent added ok");
		for (int i = 0; i < agents.size(); i++) {
			StatAgent node = (StatAgent)agents.get(i);
			EditorPlugin.DBG.info("agent "+i+" = "+node.agent.getName());
		}

		EditorPlugin.DBG.info("notifying agent listeners");
		for (int i = 0; i < agent_change_listeners.size(); i++) {
			AgentChangeListener listener = (AgentChangeListener)agent_change_listeners.get(i);

			EditorPlugin.DBG.info("notifying agent listener "+listener.getClass().getName());

			try {
				listener.agentAdded(agent);
			} catch (Throwable t) {
				EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_NOTIFY_ADD"),false);
			}	
		}
		
		updateTreeItems();
		
		return des;
	}

	public void updateModel(SDDescriptor model_root) {
//		EditorPlugin.DBG.mark("updating model - "+model_root.getName());

		OpAddToTree.launch(this,model_root,null,true);

//		EditorPlugin.DBG.mark("model updated - "+model_root.getName());

		TreeItem item = mapDescriptorToItem(model_root);
		if (item != null) {
			OpGarbageCollectTree.launch(this,item,null);
			treeItemSelected(item);
		}
	
		updateAgentImages();

		try {
			chooser_tree.redraw();
		} catch (Exception e) {}
	}

	private void addModel(SDDescriptor model_root, StatAgent statagent) {
			
		OpAddToTree.launch(this,model_root,null,false);

		statagent.model_roots.add(model_root);
//		EditorPlugin.DBG.info("model added - "+model_root.getName());

		for (int i = 0; i < model_change_listeners.size(); i++) {
			ModelChangeListener listener = (ModelChangeListener)model_change_listeners.get(i);
			listener.modelAdded(model_root);	
		}

//		EditorPlugin.DBG.info("notified listeners");

		updateAgentImages();

		try {
			chooser_tree.redraw();
		} catch (Exception e) {}
	}

	public void setAgentFile(TRCAgent trcagent, IFile ifile) {
		URI fileURI = URI.createPlatformResourceURI(ifile.getProjectRelativePath().toString());
		setAgentFile(trcagent,fileURI);		
	}

	public void setAgentFile(TRCAgent trcagent, URI model_uri) {
		StatAgent statmodel = getStatAgent(trcagent);

		EditorPlugin.DBG.info("set agent file "+trcagent.getName()+" / "+model_uri);
		
		if (statmodel != null) {
			URI uri = statmodel.uri;
			if (uri != null) {
				//model exists but they are different = dirty
				if (!uri.equals(model_uri)) {
					setDirty(true);	
				}
			} else {
				//model doesnt exist and model set to a real model = dirty
				if (model_uri != null) {
					setDirty(true);
				}		
			}

			statmodel.uri = model_uri;
		}
	}

	public void setAgentImage(TRCAgent trcagent, Image image) {
		EditorPlugin.DBG.info("set agent "+trcagent.getName()+" image");
		StatAgent agent = getStatAgent(trcagent);	
		if (agent != null) {
			agent.image = image;	
		}
		updateAgentImages();
	}

	private void updateAgentImages() {
		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);

			if (agent.agent != null) {
				TreeItem agent_item = (TreeItem)reverse_map.get(agent.agent);
				if (agent_item != null) {
					if (agent.image == null) {
						agent_item.setImage(img.getImage(ImageManager.IMG_SMODEL_TRCAGENT));
					} else {
						agent_item.setImage(agent.image);
					}		
				}
			}
				
		}		
	}
	
	public void addModelAction(TRCAgent trcagent, String action_id, String action_name, Image image, Runnable runnable) {
		StatAgent agent = getStatAgent(trcagent);
		if (agent == null) {
			//we allow adding actions before the model itself has been added
			agent = new StatAgent();
			agent.agent = trcagent;
			
			agents.add(agent);
		}
		BasicAction action = findAction(action_id,agent.modelactions);
		if (action == null) {
			action = new BasicAction();
			agent.modelactions.add(action);	
		}

		action.id = action_id;
		action.name = action_name;
		action.image = image;
		action.runnable = runnable;
	}

	public void removeModelAction(TRCAgent trcagent, String action_id) {
		StatAgent agent = getStatAgent(trcagent);
		if (agent != null) {
			for (int i = 0; i < agent.modelactions.size(); i++) {
				BasicAction action = (BasicAction)agent.modelactions.get(i);
				if (action.id.equals(action_id)) {
					agent.modelactions.remove(i);
					break;	
				}
			}
		}	
	}
	
	public void addAgentAction(TRCAgent trcagent, String action_id, String action_name, Image image, Runnable runnable) {
		StatAgent agent = getStatAgent(trcagent);
		if (agent == null) {
			//we allow adding actions before the model itself has been added
			agent = new StatAgent();
			agent.agent = trcagent;
			
			agents.add(agent);
		}
		BasicAction action = findAction(action_id,agent.actions);
		if (action == null) {
			action = new BasicAction();
			agent.actions.add(action);	
		}
		action.id = action_id;
		action.name = action_name;
		action.image = image;
		action.runnable = runnable;

	}

	public void removeAgentAction(TRCAgent trcagent, String action_id) {
		StatAgent agent = getStatAgent(trcagent);
		if (agent != null) {
			for (int i = 0; i < agent.actions.size(); i++) {
				BasicAction action = (BasicAction)agent.actions.get(i);
				if (action.id.equals(action_id)) {
					agent.actions.remove(i);
					break;	
				}
			}
		}	
	}

	public void setDirty(boolean b) {
		if (dirtiable != null) {
			dirtiable.setDirty(b);	
		}	
	}
	
	public Color parseColor(String col, Color def) {
		try {
			if (!col.startsWith("#")) throw new Exception();
			if (col.length() != 7) throw new Exception();

			String sr = col.substring(1,3);
			String sg = col.substring(3,5);
			String sb = col.substring(5,7);

			int r = Integer.parseInt(sr,16);
			int g = Integer.parseInt(sg,16);
			int b = Integer.parseInt(sb,16);

			RGB rgb = XMLConfigUtil.parseColor(col, def.getRGB());

			Color c = new Color(graph_window.getDisplay(),rgb);
			disposables.add(c);
			return c;
			
		} catch (Throwable e) {
			return def;
		}		
	}
	
	public void applyConfig(InputStream config_xml) throws Exception {
		
		try {
		
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			byte[] buf = new byte[4096];
			int n = 0;
			while (n != -1) {
				n = config_xml.read(buf,0,buf.length);
				if (n > 0) {
					bout.write(buf,0,n);	
				}	
			}
			String xml = new String(bout.toByteArray());		
			
			applyConfig(xml);
		
		} catch (Exception t) {
			EditorPlugin.DBG.warning("error applying configuration",t);
			throw t;	
		}
	}

	public void applyConfig(String config_xml) throws Exception {
		
		try {
			
			ByteArrayInputStream config_stream = new ByteArrayInputStream(config_xml.getBytes());
			Document doc = XMLConfigUtil.getDocumentBuilder().parse(config_stream);
	
			EditorPlugin.DBG.info("applying configuration");
			applyConfig(doc);
	
			EditorPlugin.DBG.info("applying configuration to all modules");
			//
			// Apply config to all modules
			//
			for (int i = 0; i < modules.size(); i++) {
				StatConModule module = (StatConModule)modules.get(i);
				module.applyConfig(config_xml);
			}

		} catch (Exception t) {
			EditorPlugin.DBG.warning("error applying configuration",t);
			throw t;	
		}
	}
	

	private void applyConfig(Document doc) throws Exception {
		
		try {
		
			Element doc_element = doc.getDocumentElement();
			
			if (doc_element.getTagName().equals("smgconfig") == false) {
				EditorPlugin.DBG.error("bad configuration - no valid root <smgconfig> element");
				throw new Exception(EditorPlugin.getString("BAD_ROOT_XML_NODE"));	
			}

			EditorPlugin.DBG.info("loading all module data");
			
			ArrayList moduledats = XMLConfigUtil.getAllElements(doc_element.getChildNodes(),"module_data");
			for (int k = 0; k < moduledats.size(); k++) {
				
				try {
				
					Element mod_dat = (Element)moduledats.get(k);
					String ref = mod_dat.getAttribute("ref");
					String data = mod_dat.getAttribute("data");
					
					StatConModule module = (StatConModule)module_refs.get(ref);
					if (module != null) {
						module.loadModuleData(HexData.hexStringToByteArray(data));
					} else {
						EditorPlugin.DBG.warning("Found data for missing module "+ref);	
					}
					
				} catch (Exception e) {
					EditorPlugin.DBG.error("Error loading data for module "+k,e);
				}
			}
			
			ArrayList xml_models = XMLConfigUtil.getAllElements(doc_element.getChildNodes(),"model");
			EditorPlugin.DBG.info("found "+xml_models.size()+" agent references");
	
			//make a first pass and load any models
			for (int i = 0; i < xml_models.size(); i++) {
				
				try {
				
					Element model = (Element)xml_models.get(i);
		
					String file = model.getAttribute("file");
		
					EditorPlugin.DBG.info("trying to load agent "+file);
		
					//get the model as a file			
	//				EditorPlugin.DBG.info("LOAD MODEL FILE "+ifile);
				
					IPath path = new Path(file); 
					IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
					IFile ifile = workspaceRoot.getFile(path);
	//EditorPlugin.DBG.info("File Path = "+ifile.getFullPath());
	//EditorPlugin.DBG.info("File Relative Path = "+ifile.getProjectRelativePath());
		
					addAgent(ifile);
	
				} catch (Throwable t) {
					EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_LOAD_CONFIG"),true);	
				}
			}
	
			ArrayList graphs = XMLConfigUtil.getAllElements(doc_element.getChildNodes(),"graph");
			EditorPlugin.DBG.info("found "+graphs.size()+" graph configurations");
			
			//make a second pass and apply any graph configurations
			for (int i = 0; i < graphs.size(); i++) {
				
				try {
				
					Element graph = (Element)graphs.get(i);
		
					if (EditorPlugin.DBG.check("configuration")) EditorPlugin.DBG.info("applying graph configuration");
		
					Color color = parseColor(graph.getAttribute("color"),graph_window.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
					boolean match_color = XMLConfigUtil.parseBoolean(graph.getAttribute("match_colors"),false);
		
					graph_window.setSlidersMatchGraph(match_color);
					graph_window.setGraphBackground(color);
		
					ArrayList ysliders = XMLConfigUtil.getAllElements(graph.getChildNodes(),"yslider");
					ArrayList xsliders = XMLConfigUtil.getAllElements(graph.getChildNodes(),"xslider");

					EditorPlugin.DBG.info("X slider count of "+xsliders.size());
					EditorPlugin.DBG.info("Y slider count of "+ysliders.size());

//this is destructive		
					//set the correct number of y sliders
//					while (graph_window.getYSliderCount() > ysliders.size()) {
//						graph_window.removeYSlider(0);
//					}
					while (graph_window.getYSliderCount() < Math.max(1,ysliders.size())) {
						graph_window.addYSlider();
					}
		
//this is destructive		
					//set the correct number of x sliders
//					while (graph_window.getXSliderCount() > xsliders.size()) {
//						graph_window.removeXSlider(0);
//					}
					while (graph_window.getXSliderCount() < Math.max(1,xsliders.size())) {
						graph_window.addXSlider();
					}
		
					for (int k = 0; k < ysliders.size(); k++) {
						Element yslider = (Element)ysliders.get(k);	
		
						String title = yslider.getAttribute("title");
						double min_limit = XMLConfigUtil.parseDouble(yslider.getAttribute("min_limit"),GraphWindow.YSLIDER_DEFMIN);
						double max_limit = XMLConfigUtil.parseDouble(yslider.getAttribute("max_limit"),GraphWindow.YSLIDER_DEFMAX);
						double min_vis = XMLConfigUtil.parseDouble(yslider.getAttribute("min_vis"),0);
						double max_vis = XMLConfigUtil.parseDouble(yslider.getAttribute("max_vis"),5000);
						double resolution = XMLConfigUtil.parseDouble(yslider.getAttribute("resolution"),0.01d);
						boolean min_locked = XMLConfigUtil.parseBoolean(yslider.getAttribute("min_locked"),false);
						boolean max_locked = XMLConfigUtil.parseBoolean(yslider.getAttribute("max_locked"),false);

						EditorPlugin.DBG.info("Configuring Y slider "+k+" "+title);
						
						ZoomSlider slider = graph_window.getYSlider(k);
						slider.setTitle(title);
						slider.lock(ZoomSlider.MIN,min_locked);
						slider.lock(ZoomSlider.MAX,max_locked);
						slider.configure(min_limit,max_limit,min_vis,max_vis,resolution);
					}
		
					for (int k = 0; k < xsliders.size(); k++) {
						Element xslider = (Element)xsliders.get(k);	
		
						String title = xslider.getAttribute("title");
						double min_limit = XMLConfigUtil.parseDouble(xslider.getAttribute("min_limit"),0);
						double max_limit = XMLConfigUtil.parseDouble(xslider.getAttribute("max_limit"),1000 * 60 * 60 * 2); //2 hours
						double min_vis = XMLConfigUtil.parseDouble(xslider.getAttribute("min_vis"),0);
						double max_vis = XMLConfigUtil.parseDouble(xslider.getAttribute("max_vis"),1000 * 60 * 3); //3 mins
						double resolution = XMLConfigUtil.parseDouble(xslider.getAttribute("resolution"),0.01d);
						boolean min_locked = XMLConfigUtil.parseBoolean(xslider.getAttribute("min_locked"),false);
						boolean max_locked = XMLConfigUtil.parseBoolean(xslider.getAttribute("max_locked"),false);
						boolean following = XMLConfigUtil.parseBoolean(xslider.getAttribute("following"),false);

						EditorPlugin.DBG.info("Configuring X slider "+k);
						
						ZoomSlider slider = graph_window.getXSlider(k);
						slider.setTitle(title);
						slider.lock(ZoomSlider.MIN,min_locked);
						slider.lock(ZoomSlider.MAX,max_locked);
						slider.configure(min_limit,max_limit,min_vis,max_vis,resolution);
						graph_window.setTimeSliderFollowing((TimeZoomSlider)slider,following);
					}
	
				} catch (Throwable t) {
					EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_APPLY_GRAPH"),true);	
				}
			}
	
			ArrayList agent_roots = XMLConfigUtil.getAllElements(doc_element.getChildNodes(),"agent");
			EditorPlugin.DBG.info("found "+agent_roots.size()+" agent/model configurations");
	
			//make a second pass and apply any descriptor configurations
			for (int i = 0; i < agent_roots.size(); i++) {
				try {
					Element agent_root = (Element)agent_roots.get(i);
					
					String rootname = agent_root.getAttribute("name");
		
//					EditorPlugin.DBG.info("looking for agent named "+rootname);
					boolean found = false;
		
					for (int k = 0; k < agents.size(); k++) { 
						StatAgent agent = (StatAgent)agents.get(k);
						if (agent.agent != null) {
							if (agent.agent.getName() != null) {
								if (agent.agent.getName().equals(rootname)) {
									EditorPlugin.DBG.info("Applying configuration to agent "+rootname+"");
									found = true;
									applyAgentConfig(agent,agent_root);	
								}
							}
						}
					}

					if (!found) {
						EditorPlugin.DBG.warning("Could not find agent "+rootname+" - unable to apply this config");
					}
	
				} catch (Throwable t) {
					EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_APPLY_MODEL"),true);	
				}
			}
	
			//
			// After we have applied the config, generate selection events on all the models
			// so that they refactor themselves accordingly
			//
			TreeItem[] items = chooser_tree.getItems();
			for (int i = 0; i < items.length; i++) {
				try {
					treeItemSelected(items[i]);
				} catch (Throwable t) {
					EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_UPDATE_MODEL"),true);	
				}
			}
	
			//
			// Update Graph Window tables
			//
			graph_window.updateTables();
		
		} catch (Exception t) {
			EditorPlugin.DBG.warning("error applying configuration",t);
			throw t;
		}

	}

	private HashMap mapChildrenByName(StatAgent agent) {
		HashMap map = new HashMap(128,0.75f);
		
		ArrayList list = agent.model_roots;
		for (int i = 0; i < list.size(); i++) {
			SDDescriptor tmp = (SDDescriptor)list.get(i);
			map.put(tmp.getName(),tmp);
		}
		
		return map;
	}

	private HashMap mapChildrenByName(TRCAgent agent) {
		HashMap map = new HashMap(128,0.75f);
		
		EList list = agent.getDescriptor();
		for (int i = 0; i < list.size(); i++) {
			SDDescriptor tmp = (SDDescriptor)list.get(i);
			map.put(tmp.getName(),tmp);
		}
		
		return map;
	}

	private HashMap mapChildrenByName(SDDescriptor descriptor) {
		HashMap map = new HashMap(128,0.75f);
		
		EList list = descriptor.getChildren();
		for (int i = 0; i < list.size(); i++) {
			SDDescriptor tmp = (SDDescriptor)list.get(i);
			map.put(tmp.getName(),tmp);
		}
		
		return map;
	}

	private void applyCBEConfig(TreeItem item, Element element, boolean parent_checked) throws Exception {

		if (element == null) return;	

		boolean ticked = XMLConfigUtil.parseBoolean(element.getAttribute("ticked"),true);
		boolean expanded = XMLConfigUtil.parseBoolean(element.getAttribute("expanded"),false);

		item.setChecked(ticked);
		item.setExpanded(expanded);

		if (parent_checked) {
			item.setGrayed(false);	
		} else {
			item.setGrayed(true);	
		}
				
		TreeItem[] items = item.getItems();
		HashMap item_map = new HashMap();
		
		for (int i = 0; i < items.length; i++) {
			item_map.put(items[i].getText(),items[i]);
		}

		ArrayList cbeconfigs = XMLConfigUtil.getAllElements(element.getChildNodes(),"cbelog");
		
		for (int i = 0; i < cbeconfigs.size(); i++) {
			Element config = (Element)cbeconfigs.get(i);
			String name = config.getAttribute("name");
			
			if (name != null) {
				TreeItem subitem = (TreeItem)item_map.get(name);
				if (subitem != null) {
					applyCBEConfig(subitem,config,parent_checked && ticked);	
				}	
			}
		}
		
	}

	private void applyAgentConfig(StatAgent agent, Element element) throws Exception {

		boolean sd_ticked = XMLConfigUtil.parseBoolean(element.getAttribute("ticked"),true);
		boolean sd_expanded = XMLConfigUtil.parseBoolean(element.getAttribute("expanded"),false);

		TreeItem item = (TreeItem)reverse_map.get(agent.agent);
		item.setChecked(sd_ticked);
		item.setExpanded(sd_expanded);

		Element cbeconfig = XMLConfigUtil.getFirstElement(element.getChildNodes(),"cbelog");

		if (cbeconfig != null) {
			applyCBEConfig(agent.msg_item,cbeconfig,sd_ticked);
		}

		ArrayList descriptor_roots = XMLConfigUtil.getAllElements(element.getChildNodes(),"descriptor");
		
		EditorPlugin.DBG.info("found "+descriptor_roots.size()+" model configurations");
		
//		EditorPlugin.DBG.info("searching for models beneath agent");

		//get a map of the descriptors under this agent
		HashMap dmap = mapChildrenByName(agent);

		//make a second pass and apply any descriptor configurations
		for (int i = 0; i < descriptor_roots.size(); i++) {
			Element model_root = (Element)descriptor_roots.get(i);
			
			String rootname = model_root.getAttribute("name");
			boolean found = false;

			SDDescriptor descriptor = (SDDescriptor)dmap.get(rootname);

			if (descriptor != null) {
				if (EditorPlugin.DBG.check("configuration")) EditorPlugin.DBG.info("Applying configuration to model "+rootname);
				try {
					applyModelConfig(descriptor,model_root,sd_ticked);	
				} catch (Throwable e) {
					EditorPlugin.DBG.error("problem applying model configuration",e);
				}
			} else {
				EditorPlugin.DBG.warning("Could not find model "+rootname+" - unable to apply this model configuration");
			}
			

		}
	}
	
	private void applyModelConfig(SDDescriptor descriptor, Element element, boolean parent_checked) throws Exception {

		boolean sd_ticked = XMLConfigUtil.parseBoolean(element.getAttribute("ticked"),true);
		boolean sd_expanded = XMLConfigUtil.parseBoolean(element.getAttribute("expanded"),false);
		
		TreeItem item = (TreeItem)reverse_map.get(descriptor);
		item.setChecked(sd_ticked);
		item.setExpanded(sd_expanded);
		
		if (parent_checked) {
			item.setGrayed(false);	
		} else {
			item.setGrayed(true);	
		}
		
		//get graph elements
		ArrayList obs_graphs = XMLConfigUtil.getAllElements(element.getChildNodes(),"obs_graph");

		//configure the graphs
		for (int i = 0; i < obs_graphs.size(); i++) {
			Element obs_graph = (Element)obs_graphs.get(i);

			boolean ticked = XMLConfigUtil.parseBoolean(obs_graph.getAttribute("ticked"),true);
			int obs_index = (int)XMLConfigUtil.parseDouble(obs_graph.getAttribute("obs_index"),0);
			int x_slider = (int)XMLConfigUtil.parseDouble(obs_graph.getAttribute("x_slider"),0);			
			int y_slider = (int)XMLConfigUtil.parseDouble(obs_graph.getAttribute("y_slider"),0);			
			Color color = parseColor(obs_graph.getAttribute("color"),graph_window.getDisplay().getSystemColor(colours[next++]));
			if (next >= colours.length) next = 0;
			int line_width = (int)XMLConfigUtil.parseDouble(obs_graph.getAttribute("line_width"),1);
			int line_style = XMLConfigUtil.parseLineStyle(obs_graph.getAttribute("line_style"),SWT.LINE_SOLID);
			double multiplier = XMLConfigUtil.parseDouble(obs_graph.getAttribute("multiplier"),1.0);
			
			int plotting_type = XMLConfigUtil.parsePlottingType(obs_graph.getAttribute("plotting_type"),Graph.PLOTTING_TYPE_AVERAGE);
			int plotting_period = XMLConfigUtil.parsePlottingPeriod(obs_graph.getAttribute("plotting_period"),Graph.PERIOD_TYPE_TICKS);
			double plotting_period_value = XMLConfigUtil.parseDouble(obs_graph.getAttribute("plotting_period_value"),1.0);
			int nodata_behaviour = XMLConfigUtil.parseNoDataBehaviour(obs_graph.getAttribute("nodata_behaviour"),Graph.NODATA_DRAW_ZERO);
			double xoffset = XMLConfigUtil.parseDouble(obs_graph.getAttribute("xoffset"),0.0);
			double yoffset = XMLConfigUtil.parseDouble(obs_graph.getAttribute("yoffset"),0.0);

			if (EditorPlugin.DBG.check("configuration")) EditorPlugin.DBG.info("Applying graph configuration for "+descriptor.getName()+" "+ticked+" "+color);
			
			if (x_slider < 0) x_slider = 0;
			if (x_slider > graph_window.getXSliderCount()) x_slider = 0;
			if (y_slider < 0) y_slider = 0;
			if (y_slider > graph_window.getYSliderCount()) y_slider = 0;
			
			if (descriptor instanceof SDMemberDescriptor) {
				SDMemberDescriptor memdescriptor = (SDMemberDescriptor)descriptor;
				EList obs_list = memdescriptor.getSnapshotObservation();
				
				if (obs_index >= 0 && obs_index < obs_list.size()) {
					
					SDSnapshotObservation observation = (SDSnapshotObservation)obs_list.get(obs_index);
				
					TreeItem obs_item = (TreeItem)reverse_map.get(observation);
					obs_item.setChecked(ticked);
					obs_item.setGrayed(!(sd_ticked && parent_checked));
				
//					if (ticked && !obs_item.getGrayed()) {
						//set up a graph over the above observation
						//
						//note - graphs etc are created right when a model is loaded, so there should always
						//be corresponding graphs for everything

						treeItemSelected(obs_item);

						BasicGraphSource source = (BasicGraphSource)source_map.get(obs_item);
						if (source != null) {

							//make sure we have a default graph to configure in the source->graph map
							Graph graph = getOrCreateGraphForSource(source);
							
//							//must use the source graph map here otherwise it might be null if the graph is not checked
//							Graph graph = (Graph)source_graph_map.get(source);
							if (graph != null) {
								
								graph.setXSlider(graph_window.getXSlider(x_slider));
								graph.setYSlider(graph_window.getYSlider(y_slider));
								graph.setForeground(color);
								graph.setLineWidth(line_width);
								graph.setLineStyle(line_style);
								graph.setStaticScaling(multiplier);

								graph.setPlottingType(plotting_type);
								graph.setPlottingPeriod(plotting_period,plotting_period_value);
								graph.setNoDataBehaviour(nodata_behaviour);
								graph.setXOffset(xoffset);
								graph.setYOffset(yoffset);
								
								
//								EditorPlugin.DBG.info("applied graph configuration for "+descriptor.getName());
	
							} else {
								EditorPlugin.DBG.warning("Could not find graph - no graph found for "+obs_item.getText());
							}
						} else {
							EditorPlugin.DBG.warning("Could not find graph - no graph source found for "+obs_item.getText());
						}

//					}
				} else {
					EditorPlugin.DBG.warning("Could not find graph - observation index out of range "+obs_index+" / "+obs_list.size());
				}
			} else {
				EditorPlugin.DBG.warning("Could not find graph - descriptor is not a member descriptor");
			}
			
		}
		
		//get the control elements
		ArrayList obs_controls = XMLConfigUtil.getAllElements(element.getChildNodes(),"control");
		
		//configure the controls
		for (int i = 0; i < obs_controls.size(); i++) {
			Element obs_control = (Element)obs_controls.get(i);
			
			boolean ticked = XMLConfigUtil.parseBoolean(obs_control.getAttribute("ticked"),true);
			int y_slider = (int)XMLConfigUtil.parseDouble(obs_control.getAttribute("y_slider"),0);			

			if (EditorPlugin.DBG.check("configuration")) EditorPlugin.DBG.info("Applying control configuration for "+descriptor.getName()+" (yslider:"+y_slider+")");

			if (descriptor instanceof SDMemberDescriptor) {
				SDMemberDescriptor memdescriptor = (SDMemberDescriptor)descriptor;
				SDRepresentation representation = memdescriptor.getRepresentation();
				
				if (representation instanceof SDModifiableVariableRepresentation) {
					SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)representation;
					
					try {
						
						TreeItem rep_item = (TreeItem)reverse_map.get(rep);
						rep_item.setChecked(ticked);
						rep_item.setGrayed(!(sd_ticked && parent_checked));

						if (y_slider >= 0 && y_slider < graph_window.getYSliderCount()) {
							bar_slider_map.put(rep_item,graph_window.getYSlider(y_slider));
						}

	//					if (ticked && !rep_item.getGrayed()) {
							//set up a control bar over the above representation
//							rep_item.setChecked(true);
//							treeItemSelected(rep_item,true);
							treeItemSelected(rep_item);
	
							ModifiableVariableBar bar = (ModifiableVariableBar)bar_map.get(rep_item);

							if (bar != null) {
								//ok for this to be null
								//may be a control with no graphical bar
								if (bar.bar != null) {
									graph_window.barMoved(bar.bar,y_slider);
								}
							} else {
								EditorPlugin.DBG.warning("Could not find bar for representation "+rep_item.getText());	
							}

						if (y_slider >= 0 && y_slider < graph_window.getYSliderCount()) {
							bar_slider_map.put(rep_item,graph_window.getYSlider(y_slider));
						}
						
	
	//					}

					} catch (NullPointerException e) {
						EditorPlugin.DBG.warning("Could not find rep item",e);
					}
				} else {
					EditorPlugin.DBG.warning("Could not find control - representation is not a modifiable variable representation");
				}
			} else {
				EditorPlugin.DBG.warning("Could not find control - descriptor is not a member descriptor");
			}
					
		}
		
		ArrayList descriptor_children = XMLConfigUtil.getAllElements(element.getChildNodes(),"descriptor");

		HashMap dmap = mapChildrenByName(descriptor);

		//recurse on the children of this descriptor
		for (int i = 0; i < descriptor_children.size(); i++) {
			Element child = (Element)descriptor_children.get(i);
			String childname = child.getAttribute("name");

			SDDescriptor child_descriptor = (SDDescriptor)dmap.get(childname);
			if (child_descriptor != null) {
				if (EditorPlugin.DBG.check("configuration")) EditorPlugin.DBG.info("Applying configuration to descriptor "+childname);
				applyModelConfig(child_descriptor,child,sd_ticked && parent_checked);	
			} else {
				EditorPlugin.DBG.warning("Could not find descriptor - descriptor of name "+childname+" was not found under descriptor "+descriptor.getName());
			}

		}

	}	

	public String generateConfig() {
		return generateConfig("");
	}

	public String generateConfig(String extra_xml) {
		StringBuffer xml = new StringBuffer();

		xml.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		xml.append("<!-- Statistical Model Graph Viewer configuration file -->\n");
		xml.append("<!-- Not intended to be human editable - edit at your own risk -->\n");
		xml.append("<smgconfig>\n");
		
		xml.append(extra_xml);
		
		for (int i = 0; i < modules.size(); i++) {
			StatConModule module = (StatConModule)modules.get(i);
			String moduleref = module.getModuleRef();
			
			try {
				byte[] moduledat = module.saveModuleData();
				String hex = HexData.byteArrayToHexString(moduledat);
				
				xml.append("<module_data ref=\""+moduleref+"\" data=\""+hex+"\" />\n");
//				xml.append(module.generateConfig());
			} catch (Exception e) {
				EditorPlugin.DBG.logVisibleError(e,EditorPlugin.getString("ERROR_SAVING_MODULE")+" "+moduleref,false);
			}
		}
		

		//
		// Append any loaded models which have URI references
		//
		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);
			URI uri = agent.uri;
			if (uri != null) {
				xml.append("<model file=\"").append(uri).append("\" />\n");
			}
		}

		//
		// Append graph configuration data
		//
		xml.append("<graph color=\"").append(XMLConfigUtil.colorToString(graph_window.getGraphBackground())).append("\" match_colors=\"").append(graph_window.getSlidersMatchGraph()).append("\" >\n");

		for (int i = 0; i < graph_window.getYSliderCount(); i++) {
			ZoomSlider slider = graph_window.getYSlider(i);
			
			xml.append("  <yslider ");
			xml.append("title=\"").append(slider.getTitle()).append("\" ");
			xml.append("min_limit=\"").append(slider.getMinLimit()).append("\" ");
			xml.append("max_limit=\"").append(slider.getMaxLimit()).append("\" ");
			xml.append("min_vis=\"").append(slider.getMinVisible()).append("\" ");
			xml.append("max_vis=\"").append(slider.getMaxVisible()).append("\" ");
			xml.append("resolution=\"").append(slider.getResolution()).append("\" ");
			xml.append("min_locked=\"").append(slider.isLocked(ZoomSlider.MIN)).append("\" ");
			xml.append("max_locked=\"").append(slider.isLocked(ZoomSlider.MAX)).append("\" ");
			xml.append("/>\n");
						
		}
		
		for (int i = 0; i < graph_window.getXSliderCount(); i++) {
			ZoomSlider slider = graph_window.getXSlider(i);

			xml.append("  <xslider ");
			xml.append("title=\"").append(slider.getTitle()).append("\" ");
			xml.append("min_limit=\"").append(slider.getMinLimit()).append("\" ");
			xml.append("max_limit=\"").append(slider.getMaxLimit()).append("\" ");
			xml.append("min_vis=\"").append(slider.getMinVisible()).append("\" ");
			xml.append("max_vis=\"").append(slider.getMaxVisible()).append("\" ");
			xml.append("resolution=\"").append(slider.getResolution()).append("\" ");
			xml.append("min_locked=\"").append(slider.isLocked(ZoomSlider.MIN)).append("\" ");
			xml.append("max_locked=\"").append(slider.isLocked(ZoomSlider.MAX)).append("\" ");
			xml.append("following=\"").append(graph_window.getTimeSliderFollowing((TimeZoomSlider)slider)).append("\" ");
			xml.append("/>\n");
			
		}

		xml.append("</graph>\n");

		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);

			generateConfig(xml,agent,0);			
			
		}
	
		xml.append("</smgconfig>\n");

		return xml.toString();
	}

	/**
	 * Generates a configuration branch for an agent.
	 * NOTE: does not include "<smgconfig>" tags - to make this a standalone 
	 * valid config file these must be added
	 * @param agent the TRCAgent in statcon to generate a configuration for
	 * @return a string containing the XML configuration file
	 */
	public String generateConfig(TRCAgent agent) {
		StringBuffer sb = new StringBuffer();
//		sb.append("<smgconfig>\n");

		StatAgent statagent = getStatAgent(agent);
		generateConfig(sb,statagent,0);

//		sb.append("</smgconfig>\n");	
		return sb.toString();
	}

	private void generateConfig(StringBuffer xml, StatAgent agent, int depth) {
		TreeItem agent_item = (TreeItem)reverse_map.get(agent.agent);

		if (agent_item == null) return;

		xml.append("<agent ");
		xml.append("name=\"").append(agent.agent.getName()).append("\" ");
		xml.append("ticked=\"").append(agent_item.getChecked()).append("\" ");
		xml.append("expanded=\"").append(agent_item.getExpanded()).append("\" ");
		xml.append(">\n");

		OpGenerateTreeConfig.launch(this,agent.msg_item,agent.msg_item,xml,"cbelog",depth+1);
		
		for (int m = 0; m < agent.model_roots.size(); m++) {
			SDDescriptor root = (SDDescriptor)agent.model_roots.get(m);
			OpGenerateDescriptorConfig.launch(this,root,root,xml,depth+1);
		}

		xml.append("</agent>");
	}


	public String generateConfig(SDDescriptor model_root) {
		try {
			StringBuffer sb = new StringBuffer();

			EditorPlugin.DBG.info("generating StatCon configuration");
			OpGenerateDescriptorConfig.launch(this,model_root,model_root,sb,1);

			return sb.toString();
		} catch (Throwable e) {
			EditorPlugin.DBG.error("problem generating StatCon configuration",e);
		}
		return null;
	}

	public GraphWindow getGraphWindow() {
		return graph_window;
	}

protected HashMap updates = new HashMap();
	public void setConstantUpdate(String request_id, boolean request_on, long ms_refresh) {
		Boolean val = new Boolean(request_on);
		updates.put(request_id,val);

		boolean on = false;
		
		Iterator requests = updates.values().iterator();
		while (requests.hasNext()) {
			Boolean b = (Boolean)requests.next();
			if (b.booleanValue()) {
				on = true;
				break;	
			}
		}
		
		if (request_on) {
			graph_window.setConstantUpdate(on,ms_refresh);
		} else {
			graph_window.setConstantUpdate(on);
		}
	}

	public void widgetDisposed(DisposeEvent e) {
		dispose();
	}

	public void dispose() {
		
		for (int a = 0; a < agents.size(); a++) {
			StatAgent sagent = (StatAgent)agents.get(a);
			TRCAgent agent = sagent.agent;
			
			EList models = agent.getDescriptor();
			for (int m = 0; m < models.size(); m++) {
				SDDescriptor model_root = (SDDescriptor)models.get(m);
				for (int i = 0; i < model_change_listeners.size(); i++) {
					ModelChangeListener listener = (ModelChangeListener)model_change_listeners.get(i);
					try {
						listener.modelRemoved(model_root);	
					} catch (Throwable e) {
						EditorPlugin.DBG.warning("failed to notify model listener "+listener+" of model remove (dispose)");
					}
				}
			}
			
			for (int i = 0; i < agent_change_listeners.size(); i++) {
				AgentChangeListener listener = (AgentChangeListener)agent_change_listeners.get(i);
				try {
		
					EditorPlugin.DBG.info("notifying agent listener "+listener.getClass().getName()+" of agent remove (dispose)");
		
					listener.agentRemoved(agent);
				} catch (Throwable e) {
					EditorPlugin.DBG.warning("failed to notify agent listener "+listener+" of agent remove (dispose)");
				}
			}
		}
		
		for (int i = 0; i < alerts.size(); i++) {
			try {
				Alert alert = (Alert)alerts.get(i);
				alert.shutdown();
			} catch (Exception e) {
				EditorPlugin.DBG.warning("failed to shut down alert");
			}
		}
		
		for (int i = 0; i < disposables.size(); i++) {
			try {
			Object o = disposables.get(i);
			if (o != null) {
				if (o instanceof Widget) {
					((Widget)o).dispose();	
				} else if (o instanceof Color) {
					((Color)o).dispose();	
				} else if (o instanceof Image) {
					((Image)o).dispose();	
				} else {
					try {
						EditorPlugin.disposeObject(o);
					} catch (Throwable e) {
						EditorPlugin.DBG.warning("dispose object failed ",e);
					}
				}
			}
			} catch (Throwable e) {
				e.printStackTrace();
			}
		}		
	}

	private void initCommonActions() {
		addDynamicAction("statcon.sliders","SNAP_ALL_XOFFSETS",EditorPlugin.getString("SNAP_ALL_XOFFSETS"),new SnapAllOffsetsDAction());
		addDynamicAction("statcon.counter","JUMP_TO_TABLE_GRAPH",EditorPlugin.getString("JUMP_TO_TABLE_GRAPH"),new JumpToGraphTableDAction());
		addDynamicAction("statcon.counter","JUMP_TO_TABLE_CONTROLS",EditorPlugin.getString("JUMP_TO_TABLE_CONTROLS"),new JumpToControlTableDAction());
		addDynamicAction("statcon.edit","AGENT_REFRESH",EditorPlugin.getString("AGENT_REFRESH"),new AgentRefreshDAction());
		addDynamicAction("statcon.edit","AGENT_SAVE",EditorPlugin.getString("AGENT_SAVE"),new AgentSaveDAction());
		addDynamicAction("statcon.edit","AGENT_REMOVE",EditorPlugin.getString("AGENT_REMOVE"),new AgentRemoveDAction());
		addDynamicAction("statcon.edit","ADD_METRIC_MONITOR",EditorPlugin.getString("ADD_METRIC_MONITOR"),new AddAlertDAction());
		addDynamicAction("statcon.edit","MERGE_OBSERVATIONS",EditorPlugin.getString("MERGE_OBSERVATIONS"),new ObservationMergeDAction());
	}
	
	private void initAsSash(Composite parent) {
		sashfolder = new SashForm(parent,0);
		sashfolder.setOrientation(SWT.HORIZONTAL);
		org.eclipse.ui.help.WorkbenchHelp.setHelp(parent,EditorPlugin.getDefault().getInfopopsId()+".statcon_anywhere");
		
		leftsash = new Group(sashfolder,0);
		rightsash = new Composite(sashfolder,0);

		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.horizontalSpacing = 0;
		layout.verticalSpacing = 0;

		leftsash.setLayout(layout);
		rightsash.setLayout(new FillLayout());

		parent.addDisposeListener(this);

		img = EditorPlugin.img;
		
		//
		// Add a test graph tab in
		//		
		initChooser(leftsash);

		GridData data = new GridData();
		data.grabExcessHorizontalSpace = true;
		data.grabExcessVerticalSpace = true;
		data.horizontalAlignment = GridData.FILL;
		data.verticalAlignment = GridData.FILL;

		chooser_tree.setLayoutData(data);

		//
		// Add the aggregate graph tab
		//
		initAggregateGraph(rightsash);

		sashfolder.setWeights(new int[]{30,70});

		disposables.add(sashfolder);
		disposables.add(leftsash);
		disposables.add(rightsash);
	}

	private void initChooser(Composite parent) {
		chooser_tree = new Tree(parent, SWT.MULTI|SWT.CHECK);
		org.eclipse.ui.help.WorkbenchHelp.setHelp(parent,EditorPlugin.getDefault().getInfopopsId()+".statcon_anywhere");
		org.eclipse.ui.help.WorkbenchHelp.setHelp(chooser_tree,EditorPlugin.getDefault().getInfopopsId()+".statcon_anywhere");

		disposables.add(chooser_tree);

		chooser_menu = new Menu(chooser_tree);
		chooser_add = new MenuItem(chooser_menu,0);

		chooser_add.setText(EditorPlugin.getString("ADD_NEW_TRACE"));
		chooser_add.setImage(img.getImage(ImageManager.IMG_SMODEL_TRCAGENT));

		disposables.add(chooser_menu);
		disposables.add(chooser_add);
		
		chooser_tree.addMouseListener(this);
		chooser_add.addSelectionListener(this);
				
		chooser_tree.addSelectionListener(this);

		DropTarget target = new DropTarget(chooser_tree,DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK);
		target.addDropListener(this);

		Transfer[] types = new Transfer[] {
//			TextTransfer.getInstance(),
//			FileTransfer.getInstance(),
			ResourceTransfer.getInstance(),
//			EditorInputTransfer.getInstance(),
//			EditorInputTransfer.getInstance(),
			};
		target.setTransfer(types);
		
		for (int i = 0; i < agents.size(); i++) {
			StatAgent agent = (StatAgent)agents.get(i);
			for (int m = 0; m < agent.model_roots.size(); m++) {
				SDDescriptor sdroot = (SDDescriptor)agent.model_roots.get(m);

				OpAddToTree.launch(this,sdroot,null,false);
			}
		}

		updateAgentImages();
	}

	private void updateTreeItems() {
		TreeItem[] items = chooser_tree.getItems();
		for (int i = 0; i < items.length; i++) {
			treeItemSelected(items[i]);
		}
	}

	public void applyStatConModelOp(StatConModelOp op, StatConOpState state, SDDescriptor start, SDDescriptor root) {
		
		//
		// get state info, then start the recursive application of the operation
		//
	
		boolean parents_checked = true;
		boolean parents_grayed = false;
	
		TreeItem parent = mapDescriptorToItem(start.getParent());
		while (parent != null) {
	
			if (!parent.getChecked()) parents_checked = false;
			if (parent.getGrayed()) parents_grayed = true;
				
			//fast fail
			if (!parents_checked && parents_grayed) break;
				
			//exit loop
			if (parent == root) break;

			parent = parent.getParentItem();
		}
	
		rapplyStatConOp(op,StatConModelOp.class,state,start,root,parents_checked, parents_grayed);		
	}

	public void applyStatConTreeOp(StatConTreeOp op, StatConOpState state, TreeItem start, TreeItem root) {
	
		//
		// get state info, then start the recursive application of the operation
		//
		boolean parents_checked = true;
		boolean parents_grayed = false;

		TreeItem parent = start.getParentItem();
		while (parent != null) {

			if (!parent.getChecked()) parents_checked = false;
			if (parent.getGrayed()) parents_grayed = true;
			
			//fast fail
			if (!parents_checked && parents_grayed) break;
			
			//exit loop
			if (parent == root) break;
			
			parent = parent.getParentItem();
		}

		rapplyStatConOp(op,StatConTreeOp.class,state,start,root,parents_checked, parents_grayed);		
	}

	private void rapplyStatConOp(StatConOp op, Class opclass, StatConOpState current_state, Object current, Object root, boolean parents_checked, boolean parents_grayed) {
		
		StatConOpState state = current_state;

		//
		// apply the operation before recusing on its children
		//
		if (opclass == StatConTreeOp.class) {
			state = ((StatConTreeOp)op).preapply(current_state, this, (TreeItem)current, (TreeItem)root, parents_checked, parents_grayed);
		} else if (opclass == StatConModelOp.class) {
			state = ((StatConModelOp)op).preapply(current_state, this, (SDDescriptor)current, (SDDescriptor)root, parents_checked, parents_grayed);
		}
		if (state == null) return;


		//
		// Get the current tree item
		//		
		TreeItem current_item = null;

		boolean cparents_checked = parents_checked;
		boolean cparents_grayed = parents_grayed;

		if (opclass == StatConTreeOp.class) {
			current_item = (TreeItem)current;

		} else if (opclass == StatConModelOp.class) {
			SDDescriptor current_des = (SDDescriptor)current;
			current_item = mapDescriptorToItem(current_des);

		} else {
			throw new ClassCastException("Unknown recursive op object type "+opclass);	
		}

		if (current_item == null) return;

		//
		// Set parent information state for children
		//
		if (!current_item.isDisposed()) {
			if (!current_item.getChecked()) cparents_checked = false;
			if (current_item.getGrayed()) cparents_grayed = true;
		}

		//
		// Apply the op to its children
		//
		if (opclass == StatConTreeOp.class) {

			//apply to children
			TreeItem[] children = ((TreeItem)current).getItems();
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					rapplyStatConOp(op,opclass,state,children[i],root,cparents_checked,cparents_grayed);	
				}
			}

		} else if (opclass == StatConModelOp.class) {

			//apply to children
			EList children = ((SDDescriptor)current).getChildren();
			if (children != null) {
				for (int i = 0; i < children.size(); i++) {
					rapplyStatConOp(op,opclass,state,(SDDescriptor)children.get(i),root,cparents_checked,cparents_grayed);	
				}
			}
		}


		//
		// apply the operation after recusing on its children
		//
		if (opclass == StatConTreeOp.class) {
			((StatConTreeOp)op).postapply(current_state, this, (TreeItem)current, (TreeItem)root, parents_checked, parents_grayed);
		} else if (opclass == StatConModelOp.class) {
			((StatConModelOp)op).postapply(current_state, this, (SDDescriptor)current, (SDDescriptor)root, parents_checked, parents_grayed);
		}
	}


	public SDDescriptor mapItemToDescriptor(TreeItem item) {
		return (SDDescriptor)sddescriptor_map.get(item);
	}

	public void mapItemToDescriptor(TreeItem item, SDDescriptor descriptor) {
		sddescriptor_map.put(item,descriptor);
	}

	public TreeItem mapAgentToItem(TRCAgent agent) {
		return (TreeItem)reverse_map.get(agent);	
	}
	
	public TreeItem mapDescriptorToItem(SDDescriptor descriptor) {
		return (TreeItem)reverse_map.get(descriptor);	
	}

	public void mapDescriptorToItem(SDDescriptor descriptor, TreeItem item) {
		reverse_map.put(descriptor,item);
		sddescriptor_map.put(item,descriptor);
	}


	public SDSnapshotObservation mapItemToObservation(TreeItem item) {
		return (SDSnapshotObservation)counter_map.get(item);
	}

	public TreeItem mapObservationToItem(SDSnapshotObservation obs) {
		return (TreeItem)reverse_map.get(obs);	
	}

	public void mapObservationToItem(SDSnapshotObservation o, TreeItem obs_item) {
		counter_map.put(obs_item,o);			
		reverse_map.put(o,obs_item);
		//not so sure about this one - means an observation treeitem is mapped to its parent descriptor
		sddescriptor_map.put(obs_item,o.getMemberDescriptor());
	}

	public SDRepresentation mapItemToRepresentation(TreeItem item) {
		return (SDRepresentation)modifiable_map.get(item);
	}

 	public TreeItem mapRepresentationToItem(SDRepresentation rep) {
		return (TreeItem)reverse_map.get(rep);	
	}

	public void mapRepresentationToItem(SDRepresentation rep, TreeItem rep_item) {
		modifiable_map.put(rep_item,rep);						
		reverse_map.put(rep,rep_item);
		//not so sure about this one - means a representation treeitem is mapped to its parent descriptor
		sddescriptor_map.put(rep_item,rep.getMemberDescriptor());
	}

	public ZoomSlider mapItemToSlider(TreeItem item) {
		return (ZoomSlider)bar_slider_map.get(item);
	}
	
	public BasicGraphSource mapItemToGraphSource(TreeItem item) {
		return (BasicGraphSource)source_map.get(item);
	}
	
	public Graph mapSourceToGraph(BasicGraphSource source) {
		return (Graph)source_graph_map.get(source);	
	}
	
	public Tree getTree() {
		return chooser_tree;	
	}

	private void selectTreePath(TreeItem leaf) {
		TreeItem tmp = leaf;
		ArrayList list = new ArrayList();
		while (tmp != null) {
			
			//this stops us adding nodes to the selection which are not visible because of closed parents
			if (!tmp.getExpanded()) list.clear();
			
			list.add(tmp);
			tmp = tmp.getParentItem();
		}
		
		TreeItem[] sel = new TreeItem[list.size()];
		list.toArray(sel);
		chooser_tree.setSelection(sel);
		
	}
	
	public void controlSelected(SDRepresentation rep) {
		TreeItem tmp = mapRepresentationToItem(rep);
		selectTreePath(tmp);
	}
	
	public void graphSelected(Graph g) {
		TreeItem tmp = (TreeItem)rgraph_map.get(g);
		selectTreePath(tmp);
	}

	public void alertSelected(Alert a) {
		TreeItem tmp = (TreeItem)reverse_map.get(a.observation);
		selectTreePath(tmp);
	}

	public void alertDeleted(Alert a) {
		alerts.remove(a);
		a.shutdown();
		getGraphWindow().setAlerts(alerts.toArray());
	}

	public void initAggregateGraph(Composite parent) {
		graph_window = new GraphWindow(parent,0,dirtiable);

		graph_window.addUpdateRunnable(new CBEUpdate());
		graph_window.addGraphWindowListener(this);
		
		disposables.add(graph_window);
	}
	
	public Control initTestGraph(Composite parent) {	
		Color black = parent.getShell().getDisplay().getSystemColor(SWT.COLOR_BLACK);
		Color cyan = parent.getShell().getDisplay().getSystemColor(SWT.COLOR_CYAN);
		Color magenta = parent.getShell().getDisplay().getSystemColor(SWT.COLOR_MAGENTA);
	
		//the execution history should be loaded ok so I can add whatever editors I want now	
		GraphWindow window = new GraphWindow(parent,0);
		disposables.add(window);
//		GraphCanvas graph_canvas = window.getGraphCanvas();
		
		window.addXSlider();
		window.addYSlider();
		
		GraphSource sine_source = new TestGraphSource(-1000000,1000000,TestGraphSource.SINE_WAVE);
		LineGraph sine_line = new LineGraph(LineGraph.PLOTTING_TYPE_AVERAGE,sine_source,window.getXSlider(0),window.getYSlider(0),cyan);
		sine_line.setLineWidth(2);
	
		GraphSource cosine_source = new TestGraphSource(-1000000,1000000,TestGraphSource.COSINE_WAVE);
		HistogramGraph cosine_line = new HistogramGraph(cosine_source,window.getXSlider(1),window.getYSlider(1),magenta);
		cosine_line.setLineWidth(2);
			
		graph_window.addGraph("Test Sine","A test sine wave graph",sine_line,null);
		graph_window.addGraph("Test Cosine","A test cosine wave graph",cosine_line,null);
		
		return window;
	}
	
	protected SDDescriptor selected_descriptor = null;
	
	public SDDescriptor getSelectedDescriptor() {
		return selected_descriptor;
	}
	
	public void setDescriptorImage(SDDescriptor descriptor, Image image) {
		TreeItem item = (TreeItem)reverse_map.get(descriptor);
		if (item == null) return;
		item.setImage(image);
	}

	//////////////////////////////
	// tree DND listener
	//////////////////////////////

	public void dragEnter(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	}
	public void dragLeave(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	} 
	public void dragOperationChanged(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	} 
	public void dragOver(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	} 
	public void dropAccept(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	}  
	public void drop(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;

		if (event.data == null) { // no data to copy, indicate failure in event.detail
			event.detail = DND.DROP_NONE;
			return;
		}
//		EditorPlugin.DBG.info("DRAG AND DROP "+event.data.getClass()+" / "+event.data);

		StringBuffer errors = new StringBuffer();

		try {
			IResource[] resources = (IResource[])event.data;

			for (int i = 0; i < resources.length; i++) {

				String path = resources[i].getFullPath().toString();
				String extension = path.substring(path.lastIndexOf(".")+1);

				EditorPlugin.DBG.info("got dragged resource "+path);

				if (extension.equalsIgnoreCase("trcmxmi") 
					|| extension.equalsIgnoreCase("trcnxmi")
					|| extension.equalsIgnoreCase("trcpxmi")
					|| extension.equalsIgnoreCase("trcaxmi")
					) {

					try {
	
						addAgent(resources[i].getFullPath());
						setDirty(true);

					} catch (Throwable e) {
						errors.append(EditorPlugin.getString("INVALID_TRACE_FILE")+":"+resources[i].getFullPath()+"\n");
						EditorPlugin.DBG.warning("bad trace/model file drop ",e);
//						e.printStackTrace();
					}
				
				} else {

					try {
						boolean accepted = false;
						
						for (int k = 0; k < drop_listeners.size(); k++) {
							DropListener drop = (DropListener)drop_listeners.get(k);
							if (drop.extension.equalsIgnoreCase(extension)) {
								if (drop.listener.drop(this,resources[i])) {
									//DROP ACCEPTED
									accepted = true;
//									return;	
								}
							}
						}
							
						if (!accepted) {
							event.detail = DND.DROP_NONE;
							throw new Exception(EditorPlugin.getString("NO_MODULE_ACCEPTED_DROP"));
						}
					
					} catch (Throwable e) {
						String msg = e.getMessage();
						if (msg == null) msg = "";
						if (msg.length() > 0) {
							errors.append(EditorPlugin.getString("INVALID_FILE")+":"+resources[i].getFullPath()+" ("+e.getMessage()+")\n");
						} else {
							errors.append(EditorPlugin.getString("INVALID_FILE")+":"+resources[i].getFullPath()+" (no further information)\n");
						}
						EditorPlugin.DBG.warning("bad module file drop "+resources[i].getFullPath(),e);
					}
				}
			}
			
		} catch (Throwable e) {
			errors.append(EditorPlugin.getString("INVALID_FILE_DROP")+":"+e+"\n");
			EditorPlugin.DBG.error("bad file drop",e);
			event.detail = DND.DROP_NONE;
			return;
		}
		if (errors.length() > 0) {
			MessageBox mbox = new MessageBox(graph_window.getShell(),SWT.OK|SWT.ICON_WARNING);	
			mbox.setText(EditorPlugin.getString("INVALID_FILE_DROP_TITLE"));
			mbox.setMessage(errors.toString());
			mbox.open();
		}
	}
	
	//////////////////////////////
	// tree popup listener
	//////////////////////////////

	private EObject getRelation(TreeItem item) {
		EObject relation = (EObject)trcagent_map.get(item);
//		if (relation == null) {
//			relation = (EObject)msglist_map.get(item);
//		}
		if (relation == null) {
			relation = (EObject)sddescriptor_map.get(item);
		}
		if (relation == null) {
			relation = (EObject)counter_map.get(item);
		}
		if (relation == null) {
			relation = (EObject)modifiable_map.get(item);
		}
		return relation;
	}
	
	Point pt = new Point(0,0);
	
	public void mouseDoubleClick(MouseEvent e) {
		Point pt = new Point(e.x,e.y);
		TreeItem item = chooser_tree.getItem(pt);
		EObject relation = getRelation(item);
		for (int i = 0; i < click_listeners.size(); i++) {
			ClickListener listener = (ClickListener)click_listeners.get(i);
			listener.doubleClick(this,item,relation,e.button);
		}
	}
	public void mouseDown(MouseEvent e) {}
	public void mouseUp(MouseEvent e) {
		Point pt = new Point(e.x,e.y);
		TreeItem item = chooser_tree.getItem(pt);
		TreeItem[] sel_items = chooser_tree.getSelection();

		ArrayList subactions = new ArrayList();
		ArrayList slideractions = new ArrayList();
			
		if (sel_items.length <= 1) {
			
			boolean toplevel = false;		
			
			if (item != null) {
				TRCAgent trcagent = (TRCAgent)trcagent_map.get(item);
				
				StatAgent agent = getStatAgent(trcagent);
				if (agent != null) {
					//item is a statistical model root node
	
					subactions = agent.actions;
					
				} else {
					//item may be a top level node - see if it has associated actions
	
					for (int k = 0; k < root_nodes.size(); k++) {
						RootNode node = (RootNode)root_nodes.get(k);
						if (node.item == item) {
							subactions = node.actions;	
							toplevel = true;
						}	
					}
					
				}
			}
			
			if (item != null && !toplevel) {
				//dont show set slider associations if the node is a top level node (a module node)
	
				for (int i = 0; i < graph_window.getYSliderCount(); i++) {
					ZoomSlider slider = graph_window.getYSlider(i);
					BasicAction action = new BasicAction();
					action.name = EditorPlugin.getString("SET_ALL_VERTICAL_TO")+" "+slider.getTitle();
					action.image = img.getImage(ImageManager.IMG_MISC_VSLIDER);
					action.runnable = new SetSliderAction(false,slider,item);
					
					slideractions.add(action);
				}
				for (int i = 0; i < graph_window.getXSliderCount(); i++) {
					ZoomSlider slider = graph_window.getXSlider(i);
					BasicAction action = new BasicAction();
					action.name = EditorPlugin.getString("SET_ALL_HORIZONTAL_TO")+" "+slider.getTitle();
					action.image = img.getImage(ImageManager.IMG_MISC_HSLIDER);
					action.runnable = new SetSliderAction(true,slider,item);
					
					slideractions.add(action);
				}
			}
		
			if (item != null) {
				SDDescriptor descriptor = (SDDescriptor)sddescriptor_map.get(item);
				selected_descriptor = descriptor;
				
				if (descriptor != null) {
	
					while (descriptor.getParent() != null) {
						descriptor = descriptor.getParent();	
					}
					
					for (int z = 0; z < agents.size(); z++) {
						StatAgent node = (StatAgent)agents.get(z);
						if (node.model_roots.contains(descriptor)) {
							subactions.addAll(node.modelactions);
							break;
						}	
					}
	
				}
			}
		}
		
		if (chooser_disposables != null) {
			for (int i = 0; i < chooser_disposables.size(); i++) {
				MenuItem mitem = (MenuItem)chooser_disposables.get(i);
				mitem.dispose();	
			}	
		}

		chooser_disposables = new ArrayList();

		if (subactions.size() > 0) {
			MenuItem sep = new MenuItem(chooser_menu,SWT.SEPARATOR);
			chooser_disposables.add(sep);	
			
			for (int k = 0; k < subactions.size(); k++) {
				BasicAction action = (BasicAction)subactions.get(k);
				action.menuitem = new MenuItem(chooser_menu,0);
				action.menuitem.setText(action.name);
				action.menuitem.setImage(action.image);
				action.menuitem.addSelectionListener(action);
				chooser_disposables.add(action.menuitem);		
			}
		}
		
		if (slideractions.size() > 0) {
			MenuItem sep = new MenuItem(chooser_menu,SWT.SEPARATOR);
			chooser_disposables.add(sep);	

			for (int k = 0; k < slideractions.size(); k++) {
				BasicAction action = (BasicAction)slideractions.get(k);
				action.menuitem = new MenuItem(chooser_menu,0);
				action.menuitem.setText(action.name);
				action.menuitem.setImage(action.image);
				action.menuitem.addSelectionListener(action);
				chooser_disposables.add(action.menuitem);		
			}
		}
		
		//
		// dynamic actions - can appear on multiply selected nodes
		//
		EObject[] sel_relations = new EObject[sel_items.length];
		
		//get the EObjects
		for (int k = 0; k < sel_items.length; k++) {
			sel_relations[k] = getRelation(sel_items[k]);
/*				
				(EObject)trcagent_map.get(sel_items[k]);
//			if (sel_relations[k] == null) {
//				sel_relations[k] = (EObject)msglist_map.get(sel_items[k]);
//			}
			if (sel_relations[k] == null) {
				sel_relations[k] = (EObject)sddescriptor_map.get(sel_items[k]);
			}
			if (sel_relations[k] == null) {
				sel_relations[k] = (EObject)counter_map.get(sel_items[k]);
			}
			if (sel_relations[k] == null) {
				sel_relations[k] = (EObject)modifiable_map.get(sel_items[k]);
			}
			*/
		}
		
		Collections.sort(dynamic_actions,new DyanmicActionComparator());

		String prev_group = null;
		boolean not_empty = false;
		
		for (int k = 0; k < dynamic_actions.size(); k++) {
			DynamicAction daction = (DynamicAction)dynamic_actions.get(k);
			
			try {
				if (daction.canAppear(this,sel_items,sel_relations)) {

					if (prev_group == null) {
						MenuItem sep = new MenuItem(chooser_menu,SWT.SEPARATOR);
						chooser_disposables.add(sep);	
					} else if (!prev_group.equals(daction.group) && not_empty) {
						MenuItem sep = new MenuItem(chooser_menu,SWT.SEPARATOR);
						chooser_disposables.add(sep);	
						not_empty = false;
					}
					prev_group = daction.group;
					
					DynamicActionWrapper action = new DynamicActionWrapper();
					action.action = daction;
					action.items = sel_items;
					action.objects = sel_relations;
	
				 	action.menuitem = new MenuItem(chooser_menu,0);
					action.menuitem.setText(daction.name);
					action.menuitem.setImage(daction.getImage());
					action.menuitem.addSelectionListener(action);
					chooser_disposables.add(action.menuitem);		
					
					not_empty = true;
				}
			} catch (Throwable t) {
				EditorPlugin.DBG.error("error in dynamic action canAppear method",t);
			}
			
		}
		
		
		if (e.button > 1) {
			chooser_menu.setVisible(true);
		}
		
		
	}	
	
	class DyanmicActionComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			DynamicAction a1 = (DynamicAction)o1;
			DynamicAction a2 = (DynamicAction)o2;
			
			int diff = a1.group.compareTo(a2.group);
			if (diff == 0) {
				return a1.name.compareTo(a1.name);
			} else {
				return diff;
			}
		}
	}
	
	//////////////////////////////
	// tree selection listener
	//////////////////////////////

	public void widgetDefaultSelected(SelectionEvent e) {
		widgetSelected(e);
	} 
	
	public void widgetSelected(SelectionEvent e) {
		Object o = e.getSource();
		widgetSelected(e.item,o);
	}

	public void widgetSelected(Widget widget, Object o) {
	
		if (o == chooser_tree) {
	
			//return on invalid tree items
			if (widget == null) return;
			if (widget instanceof TreeItem) {
	
				TreeItem item = (TreeItem) widget;
				boolean checked = item.getChecked();
				
				treeItemSelected(item);
			
			}
		
		} else if (o == chooser_add) {

			EditorPlugin.DBG.info("add agent file - opening ResourceSelectionDialog");
			
			ResourceSelectionDialog dialog = new ResourceSelectionDialog(graph_window.getShell(),ResourcesPlugin.getWorkspace().getRoot(),EditorPlugin.getString("CHOOSE_STATISTICAL_MODEL"));

			dialog.open();
			
			Object[] results = dialog.getResult();
			
			if (results != null) {
			
				for (int i = 0; i < results.length; i++) {
					EditorPlugin.DBG.info(""+results[i].getClass());
					
					if (results[i] instanceof IFile) {
						try {
							IFile file = (IFile)results[i];

							EditorPlugin.DBG.info("trying to add "+file);
							
							addAgent(file.getFullPath());	

							setDirty(true);

							chooser_tree.redraw();

						} catch (Throwable x) {
							EditorPlugin.DBG.error("model add failed",x);
						}
					}
				}
			
			}

		}

	}

protected long prev_used = 0;	
/*	
	class AgentRemoveAction implements Runnable {
		TRCAgent ragent;
		public AgentRemoveAction(TRCAgent ragent) {
			this.ragent = ragent;
		}
		public void run() {
			removeAgent(ragent);
			redoAllCBEs();
		}
	}
*/
	public void removeAgent(TRCAgent agent) {
		
		StatAgent statagent = getStatAgent(agent);
		agents.remove(statagent);
		if (statagent == null) {
			EditorPlugin.DBG.warning("remove agent found no agent (out of "+agents.size()+")");
			return;
		}

		for (int i = 0; i < agent_change_listeners.size(); i++) {
			AgentChangeListener listener = (AgentChangeListener)agent_change_listeners.get(i);
			listener.agentRemoved(agent);	
		}

		for (int m = 0; m < statagent.model_roots.size(); m++) {
			SDDescriptor model_root = (SDDescriptor)statagent.model_roots.get(m);
			for (int i = 0; i < model_change_listeners.size(); i++) {
				ModelChangeListener listener = (ModelChangeListener)model_change_listeners.get(i);
				listener.modelRemoved(model_root);	
			}
		}


//		SDDescriptor descriptor = statmodel.root;
//		models.remove(statmodel);
		
		TreeItem item = (TreeItem)reverse_map.get(agent);
		
		if (item != null) {
			// remove all the graphs and stuff associated with this model tree
			OpFreeSubTree.launch(this,item,item);
		} else {
			EditorPlugin.DBG.warning("remove agent found null tree item for agent");
		}
		
		graph_window.updateTables();

		if (item != null) {
			item.dispose();
			disposables.remove(item);
		}

		setDirty(true);

EditorPlugin.DBG.info("Reverse Map Items = "+reverse_map.values().size());

EditorPlugin.DBG.info("SDDescriptor Map Items = "+sddescriptor_map.values().size());
EditorPlugin.DBG.info("Counter Map Items = "+counter_map.values().size());
EditorPlugin.DBG.info("Modifiable Map Items = "+modifiable_map.values().size());

EditorPlugin.DBG.info("Reverse Graph Map Items = "+rgraph_map.values().size());
EditorPlugin.DBG.info("Graph Map Items = "+graph_map.values().size());
EditorPlugin.DBG.info("Bar Map Items = "+bar_map.values().size());
EditorPlugin.DBG.info("Bar Slider Map Items = "+bar_slider_map.values().size());
EditorPlugin.DBG.info("Source Map Items = "+source_map.values().size());
EditorPlugin.DBG.info("Source Graph Map Items = "+source_graph_map.values().size());

EditorPlugin.DBG.info("Disposables Items = "+disposables.size());
EditorPlugin.DBG.info("Model Items = "+agents.size());
//EditorPlugin.DBG.info("Model Roots Items = "+model_roots.size());
//EditorPlugin.DBG.info("Model Files Items = "+model_files.size());

Runtime r = Runtime.getRuntime();
r.gc();
			
//try {
//	Thread.sleep(250);	
//} catch (Exception e) {
//}
long used = r.totalMemory() - r.freeMemory();
EditorPlugin.DBG.info("USED MEMORY = "+(used/1024)+"k (+"+(used-prev_used)+")");
prev_used = used;
//try {
//	Thread.sleep(750);	
//} catch (Exception e) {
//}

	}	
	
	public void freeTreeItem(TreeItem item) {
		if (item == null) return;	
		
		Object agent = trcagent_map.remove(item);
		if (agent != null) {
			TreeItem trcagent_item = (TreeItem)reverse_map.remove(agent);
		}

		Object descriptor = sddescriptor_map.remove(item);
		if (descriptor != null) {
			TreeItem descriptor_item = (TreeItem)reverse_map.remove(descriptor);
		}

		SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)modifiable_map.remove(item);
		if (rep != null) {
			//System.out.println("REMOVING CONTROL "+rep);
			try {
				graph_window.removeControl(rep);
			} catch (Throwable e) {
				EditorPlugin.DBG.error("problem removing control from graph window",e);
			}
			TreeItem rep_item = (TreeItem)reverse_map.remove(rep);	
		}
		
		ZoomSlider rep_slider = (ZoomSlider)bar_slider_map.remove(item);
		ModifiableVariableBar bar = (ModifiableVariableBar)bar_map.remove(item);
		if (bar != null) {
			try {
				if (bar.bar.getZoomSlider() != null) {
					bar.bar.getZoomSlider().removeZoomControlBar(bar.bar);
					bar.bar.dispose();
					disposables.remove(bar.bar);
				}
			} catch (NullPointerException e) {
				EditorPlugin.DBG.warning("bar zoomslider not found",e);
			}
		}
		
		SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.remove(item);
		if (obs != null) {
			reverse_map.remove(obs);
			Graph obs_graph = (Graph)graph_map.remove(item);

			try {
				if (obs_graph != null) {
					rgraph_map.remove(obs_graph);
					obs_graph.setUseIndicator(false);
					graph_window.removeGraph(obs_graph);
				}
			} catch (Throwable e) {
				EditorPlugin.DBG.error("problem removing graph from graph window",e);
			}

			BasicGraphSource source = (BasicGraphSource)source_map.remove(item);
			if (source != null) {
				source_graph_map.remove(source);	
			}
		}

		if (!item.isDisposed()) {
			item.dispose();
		}
	}
	
	public void treeItemSelected(TreeItem item) {
		
		if (item.isDisposed()) return;
		
		try {
		
			TreeItem msgparent = item;
			while (msgparent != null) {

				if (msgparent.getText().equals(EditorPlugin.getString("MESSAGES"))) {
					redoAllCBEs();
					break;
				}
				
				msgparent = msgparent.getParentItem();
			}
		
			boolean parents_all_checked = true;
	
			TreeItem parent = item.getParentItem();
	
			while (parent != null) {
				if (parent.getChecked() == false) {
					parents_all_checked = false;
					break;	
				}
				
				parent = parent.getParentItem();
			}
			
			//preserve graying on root nodes
			if (item.getParentItem() == null) {
				boolean b = item.getGrayed();
				treeItemSelected(item, parents_all_checked);
				item.setGrayed(b);	

			} else {
				treeItemSelected(item, parents_all_checked);
			}

			graph_window.redraw();
			
		} catch (Throwable t) {
			EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_UPDATE_STATCONTREE"),false);	
		}
	}
	
	private Graph getOrCreateGraphForSource(BasicGraphSource source) {
		//make sure there is a default graph for a Graph Source
		Graph graph = (Graph)source_graph_map.get(source);

		if (graph == null) {
			
			Color col = getGraphWindow().getDisplay().getSystemColor(colours[next++]);
			
			if (next >= colours.length) {
				next = 0;
			}

			if (source instanceof GraphSource) { 
				graph = new LineGraph(LineGraph.PLOTTING_TYPE_AVERAGE,(SDSnapshotObservationGraphSource)source,graph_window.getXSlider(0),graph_window.getYSlider(0),col);
			} else if (source instanceof EnumerationGraphSource) {
				graph = new EnumerationGraph(graph_window.getXSlider(0),graph_window.getYSlider(0),col,(SDTextObservationGraphSource)source);
			}	

			source_graph_map.put(source,graph);

		}
		
		return graph;
	}
		
	public void treeItemSelected(TreeItem item, boolean parents_all_checked) {
		boolean checked = item.getChecked();

		if (!parents_all_checked) {
			checked = false;
			if (!item.getGrayed()) {
				setDirty(true);
			}
			item.setGrayed(true);
		} else {
			if (item.getGrayed()) {
//this has really bad big O - every root node we add we do like 10000 searches for it in perfmon (doh)			
//				RootNode node = getRootNodeName(item.getText());
//				if (node == null) {
					setDirty(true);
//				}
			}
			item.setGrayed(false);	
		}

		EList events = (EList)msglist_map.get(item);
		if (events != null) {
			StatAgent agent = getStatAgent(events);
			if (checked && !agent.show_messages) {
				agent.show_messages = true;
				redoAllCBEs();
				setDirty(true);
			} else if (!checked && agent.show_messages) {
				agent.show_messages = false;
				redoAllCBEs();
				setDirty(true);
			}
		} 

		//
		// is it a graphable node?
		//
		SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(item);
		if (obs != null) {

			Graph graph = (Graph)graph_map.get(item);
			BasicGraphSource source = (BasicGraphSource)source_map.get(item);
			
			//only need to do this once, no point in removing and adding sources all the time
			if (source == null) {
				
				if (obs instanceof SDContiguousObservation) {
					source = new SDSnapshotObservationGraphSource((SDContiguousObservation)obs);
				} else if (obs instanceof SDDiscreteObservation) {
					source = new SDSnapshotObservationGraphSource((SDDiscreteObservation)obs);
				} else if (obs instanceof SDTextObservation) {
					source = new SDTextObservationGraphSource((SDTextObservation)obs);
				} else {
					EditorPlugin.DBG.info("SnapshotObservation neither Contiguous nor Discrete!");
					return;
				}
				
				source_map.put(item,source);	
			}

			//
			// Create a new graph and add it
			//
			if (graph == null && checked == true) {
				setDirty(true);

				graph = getOrCreateGraphForSource(source);
/*				
				graph = (Graph)source_graph_map.get(source);

				if (graph == null) {
					
					Color col = item.getDisplay().getSystemColor(colours[next++]);
					
					if (next >= colours.length) {
						next = 0;
					}
	
					if (source instanceof GraphSource) { 
						graph = new LineGraph(LineGraph.PLOTTING_TYPE_AVERAGE,(SDSnapshotObservationGraphSource)source,graph_window.getXSlider(0),graph_window.getYSlider(0),col);
					} else if (source instanceof EnumerationGraphSource) {
						graph = new EnumerationGraph(graph_window.getXSlider(0),graph_window.getYSlider(0),col,(SDTextObservationGraphSource)source);
					}	

					source_graph_map.put(source,graph);

				}
*/
				//if x slider has been disposed, default to x slider 0
				if (graph.getXSlider().isDisposed()) {
					graph.setXSlider(graph_window.getXSlider(0));	
				}

				//if y slider has been disposed, default to x slider 0
				if (graph.getYSlider().isDisposed()) {
					graph.setYSlider(graph_window.getYSlider(0));	
				}

				SDDescriptor sd = (SDDescriptor)sddescriptor_map.get(item);
				
				graph.setUseIndicator(true);
				
				graph_window.addGraph(item.getText(),sd.getDescription(),graph,sd);

			//
			// Dispose the graph and remove it
			//
			} else if (graph != null && checked == false) {
				setDirty(true);
				
				graph.setUseIndicator(false);

				graph_window.removeGraph(graph);

			}
			
			if (checked) {
				graph_map.put(item,graph);
				rgraph_map.put(graph,item);
			} else {
				graph_map.remove(item);	
				if (graph != null) {
					rgraph_map.remove(graph);
				}
			}
			
			return;
		}
		
		//
		// is it a control node?
		//
		SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)modifiable_map.get(item);
		if (rep != null) {
			ModifiableVariableBar mbar = (ModifiableVariableBar)bar_map.get(item);
			
			//
			//create a new bar and add it
			//
			if (mbar == null && checked == true) {
				setDirty(true);

				ZoomSlider slider = (ZoomSlider)bar_slider_map.get(item);

				//if slider is null, default to Y slider 0
				if (slider == null) {
					slider = graph_window.getYSlider(0);	
				}
				
				//check to see if slider exists
				if (slider.isDisposed()) {
					slider = graph_window.getYSlider(0);	
				}

				Image enabled = rep.getImage(); 
//				if (enabled == null) {
//					enabled = img.getImage(ImageManager.IMG_SMODEL_MODIFIABLEREP);
//				}
				
				ZoomImageBar varbar = null;
				if (rep.getModifierType() != SDMeasuredVariableRepresentation.STRING && enabled != null && rep.hasBar()) {
					varbar = new ZoomImageBar(slider,(double)((Number)rep.getLastRequestedValue()).doubleValue(), enabled, enabled, enabled.getBounds().height+6);
					varbar.setToolTipText(item.getText());
					mbar = new ModifiableVariableBar(this,varbar,rep,dirtiable);
					varbar.addZoomControlBarListener(mbar);
				} else {
					//varbar==null
					mbar = new ModifiableVariableBar(this,varbar,rep,dirtiable);
				}

				SDDescriptor sd = (SDDescriptor)sddescriptor_map.get(item);
				graph_window.addControl(item.getText(),sd.getDescription(),enabled,varbar,rep);

				if (varbar != null) {
					disposables.add(varbar);
				}
				
				if (rep.getModifierType() != SDMeasuredVariableRepresentation.STRING && enabled != null && rep.hasBar()) {
					mbar.bar.getZoomSlider().addZoomControlBar(mbar.bar);
				}

			//
			//dispose the bar and remove it
			//
			} else if (mbar != null && checked == false) {
				setDirty(true);
				
				ZoomControlBar zbar = mbar.bar;
				
				try {
					bar_slider_map.put(item,zbar.getZoomSlider());
					
					zbar.dispose();
					zbar.getZoomSlider().removeZoomControlBar(zbar);
					
					disposables.remove(zbar);
				} catch (Throwable x) {
					//?
				}
				
				graph_window.removeControl(rep);
			}
			
			if (checked) {
				bar_map.put(item,mbar);
			} else {
				bar_map.remove(item);	
			}
			
			return;
		}
		
		//
		// If it is just a descriptor node
		//
		//(need to recurse down here and add/remove all the children)
		//probably easiest to get all of its children and their children in an arraylist and then call this
		//method for each of them?

		TreeItem[] children = item.getItems();

		for (int i = 0; i < children.length; i++) {
			treeItemSelected(children[i],checked);	
		}

	}  
	
	private void setTreeItemSlider(TreeItem item, boolean horizontal, ZoomSlider slider) {

		ModifiableVariableBar bar = getVariableBar(item);
		if (!horizontal) {
			bar_slider_map.put(item,slider);
			try {
				if (bar.bar.getZoomSlider() != null) {
					//move bar to other slider (if its vertical)
					bar.bar.getZoomSlider().removeZoomControlBar(bar.bar);
					bar.bar.setZoomSlider(slider);
					bar.bar.getZoomSlider().addZoomControlBar(bar.bar);
				}
			} catch (NullPointerException e) {
				EditorPlugin.DBG.warning("bar zoomslider not found",e);
			}
		}
		
		Graph obs_graph = getGraph(item);
		if (obs_graph != null) {
			if (horizontal) {
				obs_graph.setXSlider((TimeZoomSlider)slider);	
			} else {
				obs_graph.setYSlider(slider);	
			}
		}

		TreeItem[] children = item.getItems();
		for (int i = 0; i < children.length; i++) {
			setTreeItemSlider(children[i],horizontal,slider);	
		}
	}

	/////////////////////////////
	// Tree utilities
	/////////////////////////////
	
	private SDDescriptor getDescriptor(TreeItem item) {
		SDDescriptor descriptor = (SDDescriptor)sddescriptor_map.get(item);
		return descriptor;
	}
	
	private SDMemberDescriptor getMemberDescriptor(TreeItem item) {
		SDDescriptor descriptor = (SDDescriptor)sddescriptor_map.get(item);
		if (descriptor instanceof SDMemberDescriptor) {
			return (SDMemberDescriptor)descriptor;			
		} else {
			return null;
		}
	}
	
	private SDModifiableVariableRepresentation getRepresentation(TreeItem item) {
		SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)modifiable_map.get(item);
		return rep;
	}
	
	private ModifiableVariableBar getVariableBar(TreeItem item) {
		ModifiableVariableBar bar = (ModifiableVariableBar)bar_map.get(item);
		return bar;
	}

	private SDSnapshotObservation getObservation(TreeItem item) {
		SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(item);
		return obs;
	}
	
	private BasicGraphSource getGraphSource(TreeItem item) {
		BasicGraphSource obs_source = (BasicGraphSource)source_map.get(item);
		return obs_source;
	}
	
	private Graph getGraph(TreeItem item) {
		BasicGraphSource source = getGraphSource(item);
		if (source == null) return null;
		Graph obs_graph = (Graph)source_graph_map.get(source);
		return obs_graph;
	}

	
	//////////////////////////////
	// alert listener
	//////////////////////////////
	
	class AlertTableUpdate implements Runnable {
		public void run() {
			try {
				graph_window.updateAlertTable();
			} catch (Throwable t) {
				EditorPlugin.DBG.error("unable to update alert table",t);
			}
		}
	}
	
	public void alertTriggered(Alert alert) {
		Display.getDefault().asyncExec(new AlertTableUpdate());
//		graph_window.updateAlertTable();
	}
	public void alertReset(Alert alert) {
		Display.getDefault().asyncExec(new AlertTableUpdate());
//		graph_window.updateAlertTable();
	}

	abstract class TreeAction {
		public abstract void run(TreeItem item);
	}
	
	public void removeAlert(Alert alert) {
		alerts.remove(alert);
		getGraphWindow().setAlerts(alerts.toArray());
	}
	
	public void addAlert(Alert alert) {
		alert.addAlertListener(StatisticalModelGraphViewer.this);
		alerts.add(alert);
		getGraphWindow().setAlerts(alerts.toArray());
	}
	
	//////////////////////////////
	// inner classes
	//////////////////////////////


	class DropListener {
		String extension;
		StatConDropListener listener;	
	}

	class AddAlertDAction extends DynamicAction implements IInputValidator {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			if (items.length == 1) {
				SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(items[0]);
				if (obs != null) return true;
			}
			return false;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_ALERT);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(items[0]);
			Alert alert;
			
			InputDialog idialog = new InputDialog(graph_window.getShell(),EditorPlugin.getString("ALERT_NAME_TITLE"),EditorPlugin.getString("ALERT_NAME_MESSAGE"),EditorPlugin.getString("NEW_ALERT_NAME"),this);
			idialog.open();
			String name = idialog.getValue();
			if (name != null) {
	
				if (obs instanceof SDContiguousObservation) {
					alert = new Alert((SDContiguousObservation)obs);
				} else if (obs instanceof SDDiscreteObservation) {
					alert = new Alert((SDDiscreteObservation)obs);
				} else {
					EditorPlugin.DBG.warning("Alert added but no known observation type to create alert over");
					return;	
				}
				alert.name = name;
				addAlert(alert);
			}
		}		
		
		public String isValid(String s) {
			if (s.length() == 0) {
				return EditorPlugin.getString("ERROR_EMPTY_ALERT_NAME");
			}
			return null;
		}
		
	}
	
	
	class ObservationMergeDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			int descriptor_count = 0;
			int graph_count = 0;
			for (int i = 0; i < items.length; i++) {

				SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(items[i]);
				SDDescriptor des = (SDDescriptor)sddescriptor_map.get(items[i]);
				if (!(des instanceof SDMemberDescriptor)) {
					des = null;
				}
				
				if (obs != null) {
					//a graph to merge
					graph_count++;
				} else if (des != null) {
					//the destination descriptor
					descriptor_count++;
				} else {
					//dont know what to do with this thing
					return false;
				}
			}
			if (descriptor_count == 1 && graph_count > 0) {
				return true;
			} else {
				return false;
			}
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_SMODEL_COUNTER);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			ArrayList observations = new ArrayList();
			SDMemberDescriptor target = null;
			for (int i = 0; i < relations.length; i++) {

				SDSnapshotObservation obs = (SDSnapshotObservation)counter_map.get(items[i]);
				SDDescriptor des = (SDDescriptor)sddescriptor_map.get(items[i]);
				if (!(des instanceof SDMemberDescriptor)) {
					des = null;
				}

				if (obs != null) {
					//a graph to merge
					observations.add(obs);
				} else if (des != null) {
					//the destination descriptor
					target = (SDMemberDescriptor)des;
				}
			}
			
			if (target != null && observations.size() > 0) {
				ObservationMerger merger = new ObservationMerger(observations,target);
				updateModel(target);
				
				try {
					//give it a chance to recalculate (just makes it look better to the user)
					Thread.sleep(100);
				} catch (Exception e) {}
				
				getGraphWindow().updateGraphs();
				
			}
		}
	}

	class AgentSaveDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				if (!(relations[i] instanceof TRCAgent)) {
					return false;
				}
			}
			return true;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_MISC_SAVE);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				try {
					saveTRCAgent((TRCAgent)relations[i]);
				} catch (Throwable t) {
					EditorPlugin.DBG.logVisibleError(t,EditorPlugin.getString("ERROR_SAVE_GENERAL"),true);
					EditorPlugin.DBG.error("failed to save agent",t);
				}
			}
			redoAllCBEs();
		}

		public void saveTRCAgent(TRCAgent agent) throws IOException {
			try {
				agent.getAgentProxy().getProcessProxy().getNode().getMonitor().eResource().save(Collections.EMPTY_MAP);
			} catch (Exception e) {
			}
			try {
				agent.getAgentProxy().getProcessProxy().getNode().eResource().save(Collections.EMPTY_MAP);
			} catch (Exception e) {
			}
			try {
				agent.getAgentProxy().getProcessProxy().eResource().save(Collections.EMPTY_MAP);
			} catch (Exception e) {
			}
			agent.eResource().save(Collections.EMPTY_MAP);
		}
	}
	
	class AgentRefreshDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				if (!(relations[i] instanceof TRCAgent)) {
					return false;
				}
			}
			return true;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_EVENT_INVOCATION);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				try {
					updateAgent((TRCAgent)relations[i]);
				} catch (Throwable t) {
					EditorPlugin.DBG.warning("failed to remove agent",t);
				}
			}
			redoAllCBEs();
		}
	}
	class AgentRemoveDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				if (!(relations[i] instanceof TRCAgent)) {
					return false;
				}
			}
			return true;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_SMODEL_REMOVE);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				try {
					removeAgent((TRCAgent)relations[i]);
				} catch (Throwable t) {
					EditorPlugin.DBG.warning("failed to remove agent",t);
				}
			}
			redoAllCBEs();
		}
	}

	class JumpToControlTableDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			if (items.length != 1) return false;
			SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)modifiable_map.get(items[0]);
			if (rep != null) {
				if (items[0].getChecked() && !items[0].getGrayed()) {
					return true;
				} else {
					//not checked and enabled - shouldnt be in the controls view
					return false;
				}
			}
			return false;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_SMODEL_MODIFIABLEREP);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			SDModifiableVariableRepresentation rep = (SDModifiableVariableRepresentation)modifiable_map.get(items[0]);
			graph_window.showTab(GraphWindow.TAB_CONTROLS);
			graph_window.selectControl(rep);
		}
	}

	class SnapAllOffsetsDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < relations.length; i++) {
				Object o = relations[i];
				if (o instanceof SDDescriptor || o instanceof TRCAgent) {
					return true;
				}
				
			}
			return false;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_MISC_HSLIDER);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			for (int i = 0; i < items.length; i++) {
				perform(items[i]);
			} 
		}
		
		private void perform(TreeItem item) {
			Graph graph = (Graph)graph_map.get(item);
			if (graph != null) {
				double tmax = graph.getGraphSource().getMax();
				double tnow = System.currentTimeMillis();
				graph.setXOffset(tnow-tmax);
				setDirty(true);
			}
			TreeItem[] children = item.getItems();
			for (int i = 0; i <  children.length; i++) {
				perform(children[i]);
			}
		}
	}
	
	class JumpToGraphTableDAction extends DynamicAction {
		public boolean canAppear(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			if (items.length != 1) return false;
			Graph g = (Graph)graph_map.get(items[0]);
			if (g != null) {
				if (items[0].getChecked() && !items[0].getGrayed()) {
					return true;
				} else {
					//not checked and enabled - shouldnt be in the graph view
					return false;
				}
			}
			return false;
		}
		public Image getImage() {
			return img.getImage(ImageManager.IMG_SMODEL_COUNTER);
		}
		public void perform(StatConInterface statcon, TreeItem[] items, EObject[] relations) {
			Graph g = (Graph)graph_map.get(items[0]);
			graph_window.showTab(GraphWindow.TAB_GRAPHS);
			graph_window.selectGraph(g);
//			System.err.println("JUMP TO GRAPH "+g);
		}
	}
	
	class DynamicActionWrapper extends BasicAction {
		TreeItem[] items;
		EObject[] objects;
		DynamicAction action;

		public void dispose() {
			if (menuitem != null) menuitem.dispose();
		}

		public void widgetSelected(SelectionEvent e) {
			action.perform(StatisticalModelGraphViewer.this,items,objects);
		}

		public void widgetDefaultSelected(SelectionEvent e) {
			action.perform(StatisticalModelGraphViewer.this,items,objects);
		}
	}

	class BasicAction implements SelectionListener {
		String id = EditorPlugin.getString("ACTION_UNKNOWN");
		String name;
		Runnable runnable;	
		MenuItem menuitem;
		Image image;
			
		public void dispose() {
			if (menuitem != null) menuitem.dispose();
		}

		public void widgetSelected(SelectionEvent e) {
			runnable.run();
		}
		public void widgetDefaultSelected(SelectionEvent e) {
			runnable.run();
		}
	}
	
	class SetSliderAction implements Runnable {
		boolean horizontal;
		ZoomSlider slider;
		TreeItem item;
		public SetSliderAction (boolean horizontal, ZoomSlider slider, TreeItem item) {
			this.horizontal = horizontal;
			this.slider = slider;
			this.item = item;
		}
		public void run() {
			setTreeItemSlider(item,horizontal,slider);
			graph_window.redraw();
			graph_window.updateTables();
		}
	}

	class CBEComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			CBECommonBaseEvent e1 = (CBECommonBaseEvent)o1;	
			CBECommonBaseEvent e2 = (CBECommonBaseEvent)o2;
			return (int)(e1.getCreationTime() - e2.getCreationTime());	
		}	
	}

	class CBEUpdate implements Runnable {
		public void run() {
			appendAllCBEs();	
		}	
	}

	class RootNode {
		String name;
		Image image;
		ArrayList actions = new ArrayList();
		TreeItem item;
		String id;
		String parent_id;
		ArrayList node_children = new ArrayList();
		
		public void dispose() {
			item.dispose();	
		}
	}
	
	class StatAgent {
		TRCAgent agent = null;
		Image image = null;
		ArrayList actions = new ArrayList();
		ArrayList modelactions = new ArrayList();
		ArrayList model_roots = new ArrayList();
		URI uri;
		int msgindex = 0;
		boolean show_messages = true;
		
		TreeItem msg_item = null;
		TreeItem msg_info_item = null;
		TreeItem msg_warn_item = null;
		TreeItem msg_error_item = null;
		TreeItem msg_loc_item = null;
		
		HashMap msg_locations = new HashMap();
	}

}
