/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: LoopEvent.java,v 1.5 2005/02/16 22:21:34 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.test.common.event;

import org.eclipse.hyades.test.common.util.XMLUtil;

/**
 * This is a flat class packaged in the execution environment and
 * it mirrors the 
 * {@link org.eclipse.hyades.models.common.testprofile.TPFLoopEvent}.
 * 
 * @author jtoomey
 */
public class LoopEvent 
extends ExecutionEvent
{
	private boolean asychronous = false;
	private long iterations = 0;

	/**
	 * @return whether or not the loop is asychronous
	 */
	public boolean isAsychronous() {
		return asychronous;
	}

	/**
	 * @return the number of iterations in the loop
	 */
	public long getIterations() {
		return iterations;
	}

	/**
	 * @param whether or not the loop is asychronous
	 */
	public void setAsychronous(boolean b) {
		asychronous = b;
	}

	/**
	 * @param the number of iterations in the loop
	 */
	public void setIterations(long l) {
		iterations = l;
	}

	/**
	 * @see org.eclipse.hyades.test.java.event.ExecutionEvent#getXMLRoot()
	 */
	protected String getXMLRoot()
	{
		return "loopEvent";
	}

	/**
	 * @see org.eclipse.hyades.test.java.event.ExecutionEvent#addXMLAttributes(java.lang.StringBuffer)
	 */
	protected void addXMLAttributes(StringBuffer xml)
	{
		super.addXMLAttributes(xml);
		
		xml.append((XMLUtil.createXMLAttribute("iterations", Long.toString(iterations), false)).toString());
		if (asychronous)
			xml.append((XMLUtil.createXMLAttribute("asychronous", "true", false)).toString());
		else
			xml.append((XMLUtil.createXMLAttribute("asychronous", "false", false)).toString());
						

	}	

}
