/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLTraceDataProcessor.java,v 1.10 2005/04/28 16:27:51 slavescu Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.trace.ui.internal.piclient;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.io.Writer;
import java.text.DateFormat;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.hyades.internal.execution.local.common.DataServerListener;
import org.eclipse.hyades.internal.execution.local.control.Agent;
import org.eclipse.hyades.internal.execution.local.control.InactiveAgentException;
import org.eclipse.hyades.loaders.util.InvalidXMLException;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.XMLLoader;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.hierarchy.util.ModelDebugger;
import org.eclipse.hyades.trace.ui.ProfileEvent;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.ui.internal.util.PDCoreUtil;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;



public class XMLTraceDataProcessor extends XMLLoader implements
		DataServerListener {
	//~ Instance fields
	// ----------------------------------------------------------------------------

	private boolean newFile;

	private static final String NEW_LINE = "\n";

	private static final int BUFFER_SIZE = 64 * 1024;

	private boolean firstEvent = true;

	//	private File profilingFile = null;
	private OutputStream fileWriter = null;

	private String profileFileName = null;

	//	private TRCAgentProxy _agent;
	private TRCMonitor _monitor;

	private boolean _changed = false;

	private int timeout = 0;

	private String bufStr;

	private final String START_TAG = "<TRACE>";

	private final String END_TAG = "</TRACE>";

	private final String XML_VERSION_TAG = "<?xml version=\"1.0\"?>";

	private final String END_TAG_REPLACE = "\n<!-- new trace starts-->\n";
		
	private final String START_TRACE = START_TAG + NEW_LINE;

	private final String END_TRACE = END_TAG + NEW_LINE;

	private final String XML_VERSION = XML_VERSION_TAG + NEW_LINE;

	private static final char[] ID1_IN = new char[]{'i','D','1','>'};
	private static final char[] ID1_OUT = new char[]{'i','D','1','<'};
	private static final char[] ID2_IN = new char[]{'i','D','2','>'};
	private static final char[] ID2_OUT = new char[]{'i','D','2','<'};
	//~ Constructors
	// -------------------------------------------------------------------------------

	/**
	 * XMLDataProcessor constructor comment.
	 */
	public XMLTraceDataProcessor(TRCAgentProxy agent) {
		super(agent);

		setCollectionMode(agent.getCollectionMode().getValue());

		(new RefreshUI()).start();
	}

	/**
	 * XMLDataProcessor constructor comment.
	 */
	public XMLTraceDataProcessor(TRCMonitor monitor) {
		super(monitor);
	}

	//~ Methods
	// ------------------------------------------------------------------------------------

	/**
	 * Sets the fw.
	 * 
	 * @param fw
	 *            The fw to set
	 */
	public void setWriter(OutputStream fw) {
		this.fileWriter = fw;
	}

	/**
	 * Returns the fw.
	 * 
	 * @return Writer
	 */
	public OutputStream getWriter() {
		return fileWriter;
	}

	/**
	 * Sets the profileFileName.
	 * 
	 * @param profileFileName
	 *            The profileFileName to set
	 */
	public void setProfileFileName(String profileFileName) {
		this.profileFileName = profileFileName;
	}

	/**
	 * Returns the profileFileName.
	 * 
	 * @return String
	 */
	public String getProfileFileName() {
		return profileFileName;
	}

	//    /**
	//     * Sets the pFile.
	//     * @param pFile The pFile to set
	//     */
	//    public void setProfilingFile(File pFile) {
	//        this.profilingFile = pFile;
	//    }
	//
	//    /**
	//     * Returns the pFile.
	//     * @return File
	//     */
	//    public File getProfilingFile() {
	//        return profilingFile;
	//    }

	public void createWriter() {

		try {
			if (profileFileName != null) {
				fileWriter = makeWriter();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private OutputStream makeWriter() throws IOException {
		File f = new File(profileFileName);
		if (f.exists()) {
			if (!PDCoreUtil.isZipFile(profileFileName)) {
				checkAndRemoveEndTag(profileFileName);
				newFile = false;
				return new BufferedOutputStream(new FileOutputStream(
						profileFileName, true), BUFFER_SIZE);
			} else {

				File newFileName = new File(f.getParentFile().getAbsolutePath()
						+ "\\Temp-" + (new Date()).getTime());
				renameFile(f, newFileName);

				ZipOutputStream zipOutputStream = new ZipOutputStream(
						new BufferedOutputStream(new FileOutputStream(f),
								BUFFER_SIZE));
				ZipInputStream zipInputStream = new ZipInputStream(
						new BufferedInputStream(
								new FileInputStream(newFileName), BUFFER_SIZE));

				copyZip(zipInputStream, zipOutputStream);

				zipInputStream.close();

				newFileName.delete();

				ZipEntry traceEntry = new ZipEntry("TraceContents-"
						+ getTimeStamp() + ".xml");
				zipOutputStream.putNextEntry(traceEntry);

				newFile = true;

				return zipOutputStream;
			}
		} else {
			OutputStream fos = new BufferedOutputStream(new FileOutputStream(
					profileFileName));
			ZipOutputStream zipStream = new ZipOutputStream(fos);
			ZipEntry traceEntry = new ZipEntry("TraceContents-"
					+ getTimeStamp() + ".xml");
			zipStream.putNextEntry(traceEntry);
			newFile = true;

			return zipStream;
		}
	}

	private boolean renameFile(File f, File newFileName) {
		boolean ret = f.renameTo(newFileName);
		if (ret == false) {
			ret = copyFile(f, newFileName);
			f.delete();

		}
		return ret;
	}

	/**
	 * @param f
	 * @param newFileName
	 */
	private boolean copyFile(File f, File newFileName) {
		try {
			BufferedInputStream bufferedInputStream = new BufferedInputStream(
					new FileInputStream(f), BUFFER_SIZE);
			BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(
					new FileOutputStream(newFileName), BUFFER_SIZE);
			byte[] buffer = new byte[BUFFER_SIZE];
			while (bufferedInputStream.available() > 0) {
				int bytesRead = bufferedInputStream.read(buffer);
				bufferedOutputStream.write(buffer);
			}
			bufferedInputStream.close();
			bufferedOutputStream.close();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * @param zipInputStream
	 * @param zipStream
	 */
	private void copyZip(ZipInputStream zipInputStream,
			ZipOutputStream zipOutputStream) throws IOException {
		ZipEntry zipEntry;
		int byteCount;
		byte[] data = new byte[BUFFER_SIZE];
		while ((zipEntry = zipInputStream.getNextEntry()) != null) {
			zipOutputStream.putNextEntry(zipEntry);
			while ((byteCount = zipInputStream.read(data, 0, BUFFER_SIZE)) != -1) {
				zipOutputStream.write(data, 0, byteCount);
			}
			zipOutputStream.closeEntry();
		}
	}

	/**
	 * @return
	 */
	private String getTimeStamp() {
		return DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL)
				.format(new Date()).replace(' ', '_').replace(':', '_')
				.replace(',', '_');
	}

	public void dataServerExited() {
		try {
			if (getAgentProxy() != null) {
				_changed = false;
				getAgentProxy().setCollectionData(false);
				endTrace(getWriter());
				setProfileFileName(null);
				//                setProfilingFile(null);
				setWriter(null);

				cleanUp();

				Display d = Display.getDefault();

				d.asyncExec(new Runnable() {
					public void run() {
						ProfileEvent event = UIPlugin.getDefault()
								.getProfileEvent();

						event.setSource(getAgentProxy());
						event.setType(ProfileEvent.STOP_COLLECTING);
						UIPlugin.getDefault().notifyProfileEventListener(event);
					}
				});
			}
		} catch (Exception exc) {
			//do nothing
		}
	}

	public void dispose() {
		super.cleanUp();

		fileWriter = null;
	}

	public void endTrace(OutputStream randomFile) {
		try {
			if (randomFile != null) {
					            randomFile.write(END_TRACE.getBytes());
					            randomFile.flush();
				randomFile.close();
				randomFile = null;
			}
		} catch (IOException e) {
		}
	}

	private boolean isValidTag(byte[] buf) {
		if (startsWith(buf, START_TAG.getBytes())
				|| startsWith(buf, XML_VERSION_TAG.getBytes()))
			return false;
		else
			return true;

	}

	/**
	 * @param container
	 * @param bytes
	 * @return
	 */
	private boolean startsWith(byte[] container, byte[] bytes) {
		if (container.length < bytes.length)
			return false;
		for (int i = 0; i < bytes.length; i++) {
			if (container[i] != bytes[i]) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Insert the method's description here. Creation date: (3/1/01 12:06:07 PM)
	 * 
	 * @param buffer
	 *            byte[]
	 * @param length
	 *            int
	 * @param peer
	 *            java.net.InetAddress
	 */
	public void incommingData(byte[] buffer, int length,
			java.net.InetAddress peer) {
		_changed = true;
		if(ModelDebugger.INSTANCE.debugEventFlow)
		{
		System.out.println(ID1_IN);
		}
		try {
			if ((getAgentProxy() != null) && getAgentProxy().isToProfileFile()) {
				//            	bufStr=new String(buffer, 0, length, "UTF8");
				if (isValidTag(buffer)) {
					writeByte(buffer, length);
				}
			} else {
				if (firstEvent) {
					if (length > 1 && !(buffer[0] == '<' && buffer[1] == '?')) {
						LoadersUtils.loadRootEvent(this);
					}
					firstEvent = false;
				}
				super.loadEvent(buffer, length);
			}
		} catch (InvalidXMLException e) {
		} catch (java.lang.OutOfMemoryError e) {
			handleOutOfMemoryError();
			stopAgent();
			if(ModelDebugger.INSTANCE.debugEventFlow)
			{
			System.out.print(ID1_OUT);
			System.out.println(" = "+e.getLocalizedMessage());
			}
			LoadersUtils.log(e);
			throw e;
		}
		if(ModelDebugger.INSTANCE.debugEventFlow)
		{
		System.out.println(ID1_OUT);
		}
	}

	/**
	 * Insert the method's description here. Creation date: (3/1/01 12:06:07 PM)
	 * 
	 * @param buffer
	 *            char[]
	 * @param length
	 *            int
	 * @param peer
	 *            java.net.InetAddress
	 */
	public void incommingData(char[] buffer, int length,
			java.net.InetAddress peer) {
		if(ModelDebugger.INSTANCE.debugEventFlow)
		{
		System.out.println(ID2_IN);
		}
		byte[] newBuffer = new byte[length];

		for (int i = 0; i < length; i++) {
			newBuffer[i] = (byte) buffer[i];
		}

		try {
			if ((getAgentProxy() != null) && getAgentProxy().isToProfileFile()) {
				//				bufStr=new String(newBuffer, "UTF8");
				if (isValidTag(newBuffer)) {
					writeByte(newBuffer, length);
				}

				//new emf
				//                super.loadEvent(newBuffer, length, false, true);
			} else {
				if (firstEvent) {
					if (length > 1 && !(buffer[0] == '<' && buffer[1] == '?')) {
						LoadersUtils.loadRootEvent(this);
					}
					firstEvent = false;
				}
				super.loadEvent(newBuffer, length);
			}
		} catch (InvalidXMLException e) {
		} catch (java.lang.OutOfMemoryError e) {
			if(ModelDebugger.INSTANCE.debugEventFlow)
			{
			System.out.print(ID2_OUT);
			System.out.println(" = "+e.getLocalizedMessage());
			}
			handleOutOfMemoryError();
			stopAgent();
			LoadersUtils.log(e);
			throw e;
		}
		System.out.println(ID2_OUT);
	}

	/**
	 * Insert the method's description here. Creation date: (3/1/01 12:06:07 PM)
	 * 
	 * @param data
	 *            byte[]
	 * @param length
	 *            int
	 * @param peer
	 *            java.net.InetAddress
	 */
	public void invalidDataType(byte[] data, int length,
			java.net.InetAddress peer) {
	}

	/**
	 * Insert the method's description here. Creation date: (3/1/01 12:06:07 PM)
	 */
	public void waitingForData() {
	}

	public void startTrace(OutputStream randomFile) {
		try {
			if (getAgentProxy() != null) {
				randomFile.write(START_TRACE.getBytes());
				randomFile.flush();
			}
		} catch (IOException e) {
		}
	}

	public void writeHeader(Writer randomFile) {
		// // try {
		//            if (getAgentProxy() != null) {
		//                startTrace(randomFile);
		//// writeNode(randomFile);
		//// writeProcessCreate(randomFile);
		//// writeAgentCreate(randomFile);
		//// writeTraceStart(randomFile);
		//// writeFilter(randomFile);
		//// writeOption(randomFile);
		//            }
		//// } catch (IOException e) {
		//// }
	}

	/*
	 * Write the necessary trace event to the head to the head of the file
	 */
	public void writeXMLVersion(OutputStream randomFile) {
		try {
			randomFile.write(XML_VERSION.getBytes());
			randomFile.flush();
		} catch (IOException e) {
		}
	}

	/**
	 *  
	 */
	private TRCAgentProxy getAgentProxy() {
		return getContext().getAgentProxy();
	}

	private String getApplicationExecutableString(TRCProcessProxy process) {
		String exeStr = "";

		if (process.getName() == null) {
			return "";
		}

		if (!process.getName().equals("")) {
			exeStr = "java.exe -XrunpiAgent:server=controlled ";

			if ((process.getClasspath() != null)
					&& !process.getClasspath().equals("")) {
				exeStr += "-cp ";
				exeStr += process.getClasspath().trim();
				exeStr += " ";
			}

			if ((process.getVmArguments() != null)
					&& !process.getVmArguments().equals("")) {
				exeStr += process.getVmArguments().trim();
				exeStr += " ";
			}

			exeStr += process.getName();

			if ((process.getParameters() != null)
					&& !process.getParameters().equals("")) {
				exeStr += process.getParameters();
			}
		}

		return exeStr;
	}

	private void handleOutOfMemoryError() {
		/* Inform the user we are running out of memory */
		String text = UIPlugin
				.getResourceString("PROFILE_OUT_OF_MEMORY_ERROR_");
		final Status err = new Status(Status.WARNING,
				ResourcesPlugin.PI_RESOURCES, IResourceStatus.INTERNAL_ERROR,
				text, null);

		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				ErrorDialog.openError(new Shell(), UIPlugin
						.getResourceString("TRACE_MSG"), UIPlugin
						.getResourceString("OUT_OF_MEMORY_ERROR_TITLE"), err);
			}
		});
	}

	private synchronized void notifyListener(Agent a, TRCAgentProxy agent) {
		final TRCAgentProxy ag = agent;

		Display d = Display.getDefault();

		d.asyncExec(new Runnable() {
			public void run() {
				//update ui
				ProfileEvent event = UIPlugin.getDefault().getProfileEvent();

				event.setSource(ag);
				event.setType(ProfileEvent.STOP_MONITOR);
				UIPlugin.getDefault().notifyProfileEventListener(event);
			}
		});
	}

	private void stopAgent() {
		if (getAgentProxy() != null) {

			try {
				Agent ai = (Agent) LoadersUtils
						.locateAgentInstance(getAgentProxy());
				if (ai != null)
					ai.stopMonitoring();
			} catch (InactiveAgentException e) {
				e.printStackTrace();
			}
			getAgentProxy().setMonitored(false);
			getAgentProxy().setAttached(false);
			getAgentProxy().setActive(false);

			LoadersUtils.deregisterDataProcessor(getAgentProxy());

			if (fileWriter != null) {
				endTrace(fileWriter);
			}

			notifyListener((Agent) LoadersUtils
					.locateAgentInstance(getAgentProxy()), getAgentProxy());
		}
	}

	//    private void writeAgentCreate(Writer randomFile) throws IOException {
	//        if (getAgentProxy() != null) {
	//            randomFile.write(new StringBuffer("<agentCreate
	// agentId=\"").append(getAgentProxy().getRuntimeId()).append("\"
	// version=\"1.000").append("\"
	// processIdRef=\"").append(getAgentProxy().getProcessProxy().getRuntimeId()).append("\"
	// agentName=\"").append(getAgentProxy().getName()).append("\"
	// agentType=\"").append(getAgentProxy().getType()).append("\"
	// agentParameters=\"").append((getAgentProxy().getProcessProxy().getLaunchMode()
	// == 0) ? "server=controlled" : "").append("\" time=\"").append(new
	// Date().getTime() / 1000).append("\"/>\n").toString());
	//            randomFile.flush();
	//        }
	//    }

	private void writeByte(byte[] data, int length) {

		try {
			if (fileWriter != null) {
				fileWriter.write(data, 0, length);
				fileWriter.write(NEW_LINE.getBytes());
				fileWriter.flush();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	//    private void writeConfigFilter(EList filters, Writer randomFile) throws
	// IOException {
	//        if (filters != null) {
	//            Iterator i = filters.iterator();
	//
	//            while (i.hasNext()) {
	//                TRCFilter filter = (TRCFilter) i.next();
	//                String method = filter.getOperation();
	//
	//                randomFile.write(new StringBuffer("<filter
	// pattern=\"").append(filter.getPattern()).append("\"
	// mode=\"").append(filter.getMode()).append("\"
	// genericPattern=\"SUFFIX\"/>\n").toString());
	//                randomFile.flush();
	//            }
	//        }
	//    }
	//
	//    private void writeConfigOption(EList options, Writer randomFile) throws
	// IOException {
	//        if (options != null) {
	//            Iterator it = options.iterator();
	//
	//            while (it.hasNext()) {
	//                TRCOption option = (TRCOption) it.next();
	//
	//                randomFile.write(new StringBuffer("<option
	// key=\"").append(option.getKey()).append("\"
	// value=\"").append(option.getValue()).append("\"/>\n").toString());
	//                randomFile.flush();
	//            }
	//        }
	//    }

	//    private void writeFilter(Writer randomFile) throws IOException {
	//        if (getAgentProxy() != null) {
	//            EList filters = null;
	//
	//            EList configs = getAgentProxy().getConfigurations();
	//            Iterator i = configs.iterator();
	//
	//            if (i.hasNext()) {
	//                TRCConfiguration config = (TRCConfiguration) i.next();
	//
	//                filters = config.getFilters();
	//
	//                writeConfigFilter(filters, randomFile);
	//
	//                return;
	//            }
	//
	//            writeConfigFilter(filters, randomFile);
	//        }
	//    }

	//    private void writeNode(Writer randomFile) throws IOException {
	//        if (getAgentProxy() != null) {
	//            TRCNode node = getAgentProxy().getProcessProxy().getNode();
	//
	//            if (node == null) {
	//                return;
	//            }
	//
	//            randomFile.write(new StringBuffer("<node
	// nodeId=\"").append(getAgentProxy().getProcessProxy().getNode().getRuntimeId()).append("\"
	// hostname=\"").append(node.getName()).append("\"
	// ipaddress=\"").append((node.getIpAddress() != null) ? node.getIpAddress()
	// : "-Unavailable-").append("\"
	// timezone=\"").append(node.getTimezone()).append("\" time=\"").append(new
	// Date().getTime() / 1000).append("\"/>\n").toString());
	//            randomFile.flush();
	//        }
	//    }

	//    private void writeOption(Writer randomFile) throws IOException {
	//        EList options = null;
	//
	//        EList configs = getAgentProxy().getConfigurations();
	//        Iterator i = configs.iterator();
	//
	//        if (i.hasNext()) {
	//            TRCConfiguration config = (TRCConfiguration) i.next();
	//
	//            options = config.getOptions();
	//
	//            writeConfigOption(options, randomFile);
	//        }
	//    }

	//    private void writeProcessCreate(Writer randomFile) throws IOException {
	//        if (getAgentProxy() != null) {
	//            TRCProcessProxy processCreate = getAgentProxy().getProcessProxy();
	//
	//            if (processCreate == null) {
	//                return;
	//            }
	//
	//            randomFile.write(new StringBuffer("<processCreate
	// processId=\"").append(processCreate.getRuntimeId()).append("\" pid=\"" +
	// processCreate.getPid()).append("\"
	// nodeIdRef=\"").append(processCreate.getNode().getRuntimeId()).append("\"
	// time=\"").append(new Date().getTime() / 1000).append("\"").append("
	// application_executable=\"").append(getApplicationExecutableString(processCreate)).append("\"/>\n").toString());
	//            randomFile.flush();
	//        }
	//    }

	//    private void writeTraceStart(Writer randomFile) throws IOException {
	//        if (getAgentProxy() != null) {
	//            randomFile.write(new StringBuffer("<traceStart traceId=\"").append("\"
	// agentIdRef=\"").append(getAgentProxy().getRuntimeId()).append("\"
	// time=\"\"/>\n").toString());
	//            randomFile.flush();
	//        }
	//    }

	//~ Inner Classes
	// ------------------------------------------------------------------------------

	class RefreshUI extends Thread {
		public RefreshUI() {
			super("Profile UI");
		}

		public void run() {
			while ((getAgentProxy() != null) && getAgentProxy().isActive()) {
				try {
					if (_changed) {
						_changed = false;

						if (!getAgentProxy().isCollectionData()) {
							getAgentProxy().setCollectionData(true);

							Display d = Display.getDefault();

							d.asyncExec(new Runnable() {
								public void run() {
									ProfileEvent event = UIPlugin.getDefault()
											.getProfileEvent();

									event.setSource(getAgentProxy());
									event.setType(ProfileEvent.COLLECTING);
									UIPlugin.getDefault()
											.notifyProfileEventListener(event);
								}
							});
						}
					} else {
						if (getAgentProxy().isCollectionData()) {
							getAgentProxy().setCollectionData(false);

							Display d = Display.getDefault();

							d.asyncExec(new Runnable() {
								public void run() {
									ProfileEvent event = UIPlugin.getDefault()
											.getProfileEvent();

									event.setSource(getAgentProxy());
									event.setType(ProfileEvent.STOP_COLLECTING);
									UIPlugin.getDefault()
											.notifyProfileEventListener(event);
								}
							});
						}
					}

					sleep(1000);
				} catch (InterruptedException exc) {
				}
			}
		}
	}

	/**
	 * @return
	 */
	public boolean isNewProfileFile() {
		return newFile;
	}

	/** 
	 * Check the non-zip format profiling file for an trace end tag and remove it
	 * if there exist one. 
	 * @param filename  
	 * 		file to be checked
	 * @return true if end tag exists and removed, false if nothing change.
	 */
	public boolean checkAndRemoveEndTag(String filename)
	{
		File checkFile = new File(filename);
		if (checkFile.exists() && checkFile.isFile()) 
		{	
			try {
				RandomAccessFile raf = new RandomAccessFile(checkFile,"rw");
				long checkFileSize = raf.length();
				StringBuffer b = new StringBuffer(END_TAG);
				//Assuming profile file is not modified manually and end tag is 
				// always the last entry of the file. 
				long seekLocation = checkFileSize - b.length()-2;
				if ( seekLocation<0) 
					seekLocation=0;
				raf.seek(seekLocation);
				while (seekLocation<=checkFileSize) {
					String readLine = raf.readLine();
					StringBuffer t = new StringBuffer(readLine);
					if (readLine.indexOf(END_TAG)>=0)
					{
						raf.seek(seekLocation);
						//mark the new trace
						raf.writeBytes(END_TAG_REPLACE);
						raf.close();
						return true;
					}
					seekLocation = seekLocation + t.length();
				}
				raf.close();
			}
			catch (Exception e)	
			{
				e.printStackTrace(); 
			}
		}
		return false;
	}
}
