/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: SDViewerPlugin.java,v 1.15 2005/03/11 18:54:35 dguilbaud Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.uml2sd.ui;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.hyades.uml2sd.ui.load.BackgroundLoader;
import org.eclipse.hyades.uml2sd.ui.load.IUml2SDLoader;
import org.eclipse.hyades.uml2sd.ui.load.LoadersManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class SDViewerPlugin extends AbstractUIPlugin {

	public static final String LOADER_TAG = "uml2SDLoader"; //$NON-NLS-1$
	public static final String LOADER_PREFIX = LOADER_TAG+"."; //$NON-NLS-1$
	
	//The shared instance.
	private static SDViewerPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public SDViewerPlugin()
   {
		super();
		plugin = this;
	}
	
	public void start(BundleContext context) throws Exception {
		super.start(context);
		try {
			this.resourceBundle = Platform.getResourceBundle(this.getBundle());
		} catch (MissingResourceException x) {
			this.resourceBundle = null;
		}
		
		// Test code:
//		List views = getSavedViews();
//		for (Iterator i = views.iterator(); i.hasNext(); ) {
//			String viewId = (String)i.next();
//			String lastLoader = getSavedLoader(viewId);
//			List loaders = getLoaders(viewId);
//			for (Iterator j = loaders.iterator(); j.hasNext(); ) {
//				IConfigurationElement loader = (IConfigurationElement)j.next();
//			}
//		}
	}
	
	public void stop(BundleContext context) throws Exception {
		BackgroundLoader.shutdown();
		super.stop(context);
	}
	
//	public List getSavedViews() {
//		ArrayList ret = new ArrayList();
//		Preferences p = getPluginPreferences();
//		String[] loaders = p.propertyNames();
//		for (int h = 0; h < loaders.length; h++) {
//			if (loaders[h].startsWith(LOADER_PREFIX)) {
//				String viewId = loaders[h].substring(LOADER_PREFIX.length());
//				if (!ret.contains(viewId)) {
//					ret.add(viewId);
//				}
//			}
//		}
//		return ret;
//	}
	
	private static HashMap viewLoadersList = new HashMap();
	
	public static List getLoaders(String viewId) {
		List list = (List)viewLoadersList.get(viewId);
		if (list != null) {
			return list;
		}
		ArrayList ret = new ArrayList();
		IExtensionPoint iep = Platform.getExtensionRegistry().getExtensionPoint(getId(), LOADER_TAG);
		IExtension ie[] = iep.getExtensions();
		for (int i = 0; i < ie.length; i++) {
			IConfigurationElement c[] = ie[i].getConfigurationElements();
			for (int j = 0; j < c.length; j++) {
				if (viewId.equals(c[j].getAttribute("view"))) { //$NON-NLS-1$
					ret.add(c[j]);
				}
			}
		}
		viewLoadersList.put(viewId, ret);
		return ret;
	}

	/**
	 * @param viewId
	 * @param ce
	 */
	public static void createLoaderForView(String viewId, IConfigurationElement ce) {
		try {
			Object obj = ce.createExecutableExtension("class"); //$NON-NLS-1$
			IUml2SDLoader l = (IUml2SDLoader) obj;
			if (viewId != null)
				LoadersManager.getLoadersManager().setCurrentLoader(l, viewId); //$NON-NLS-1$
		} catch (CoreException e4) {
			System.err.println("Error 'uml2SDLoader' Extension point :" + e4); //$NON-NLS-1$
		} catch (Exception e5) {
			e5.printStackTrace();
			System.err.println("Error 'uml2SDLoader' Extension point :" + e5); //$NON-NLS-1$
		}
	}

	/**
	 * Returns the loader class name that have been saved last time
	 * @param viewId the view this loader belongs to
	 * @return the class name of the saved loader
	 */
	public String getSavedLoader(String viewId) {
		Preferences p = getPluginPreferences();
		return p.getString(LOADER_PREFIX+viewId);
	}
	
	/**
	 * Saves the last loader in order to reload it on next session.
	 */
	public void setLastLoader(String id,String id2) {
		Preferences p = getPluginPreferences();
		p.setValue(LOADER_PREFIX+id2, id);
	}
	
	/**
	 * Returns the shared instance.
	 */
	public static SDViewerPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= SDViewerPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}


	private static URL BASIC_URL=null;
	static
	{
		//URL to get images from plugins
		//perhaps we can found better code to load images ...
		//but at this time I know this: 
		try {
		BASIC_URL = new URL( "platform", "localhost", "plugin");//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} catch( Exception E ) {
			System.err.println(E);
		}
	}

	public static ImageDescriptor getResourceImage( String _name )
	{
		try {
		 URL url=new URL( BASIC_URL, "plugin/org.eclipse.hyades.uml2sd.ui/icons/"+_name);//$NON-NLS-1$
		 ImageDescriptor img = ImageDescriptor.createFromURL( url );
		return img ;
		} catch( Exception E ) {
					 System.err.println(E);
		}
		return null;
	}
	
	public static String getId()
	{
		return getDefault().getBundle().getSymbolicName(); 
	}
	
	public static boolean debugIndex()
	{
		if (debug())
		{
			if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/updateindex").//$NON-NLS-1$
				equalsIgnoreCase("true"))//$NON-NLS-1$
				return true;
		}
		return false;
	}
	
	public static boolean debugDisplay()
	{
		if (debug())
		{
			if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/display").//$NON-NLS-1$
				equalsIgnoreCase("true"))//$NON-NLS-1$
				return true;
		}
		return false;
	}
	
	public static boolean debugDoubleBuffer()
	{
		if (debug())
		{
			if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/doublebuffer").//$NON-NLS-1$
				equalsIgnoreCase("true"))//$NON-NLS-1$
				return true;
		}
		return false;
	}
	
	public static boolean debugSorting()
	{
		if (debug())
		{
			if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/sorting").//$NON-NLS-1$
				equalsIgnoreCase("true"))//$NON-NLS-1$
				return true;
		}
		return false;
	}
	
	public static boolean debug()
	{
		if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/debug")!=null) //$NON-NLS-1$
		{
			if (Platform.getDebugOption("org.eclipse.hyades.uml2sd.ui/debug").//$NON-NLS-1$
				equalsIgnoreCase("true")) //$NON-NLS-1$
				return true; 
		}
		return false;
	}
	
}
